﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IOTransmissionAbutment
{
    public enum DeviceType { None,PowerStation, Gate, Elevator,Roller,Other }
    public abstract class Device
    {
        private int _ID;
        private string _Name;
        private string _IPAddress;
        private DeviceType _Type;
        private DateTime _CreationTime;
        private DateTime _ChangingTime;
        private string _Remark;
        private byte[] _ReceiveBytes;
        private byte[] _SendBytes;
        private bool _Online;
        private int _Carrier;

        public Device()
        {
            _SendBytes = new byte[14] { 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03 };
            _SendBytes[12] = Xor(_SendBytes);
        }
        protected internal virtual void InitSendBytes()
        {
            if (_SendBytes != null && _SendBytes.Length == 14)
            {
                switch (_Type)
                {
                    case DeviceType.Gate:
                        _SendBytes[1] = 0x02;
                        break;
                    case DeviceType.Elevator:
                        _SendBytes[1] = 0x03;
                        break;
                    case DeviceType.PowerStation:
                        _SendBytes[1] = 0x01;
                        break;
                    case DeviceType.Roller:
                        _SendBytes[1] = 0x04;
                        break;
                    case DeviceType.Other:
                        _SendBytes[1] = 0x80;//0x05
                        break;
                    default:
                        break;
                }
                _SendBytes[2] = (byte)_ID;
                _SendBytes[12] = Xor(_SendBytes);
            }
        }
        public virtual bool Online
        {
            get
            {
                return _Online;
            }

            set
            {
                _Online = value;
            }
        }

        public virtual int Carrier
        {
            get
            {
                return _Carrier;
            }

            set
            {
                if (_Carrier != value)
                {
                    _Carrier = value;
                    if (_Carrier == 0)
                    {
                        ResetCtrl();
                    }
                }
            }
        }
        public virtual byte[] ReceiveBytes
        {
            get
            {
                return _ReceiveBytes;
            }

            set
            {
                _ReceiveBytes = value;
            }
        }
        public virtual byte[] SendBytes
        {
            get
            {
                return _SendBytes;
            }

            set
            {
                _SendBytes = value;
            }
        }
        public virtual int ID
        {
            get
            {
                return _ID;
            }

            set
            {
                _ID = value;
            }
        }


        public virtual string Name
        {
            get
            {
                return _Name;
            }

            set
            {
                _Name = value;
            }
        }
        public virtual string IPAddress
        {
            get
            {
                return _IPAddress;
            }

            set
            {
                _IPAddress = value;
            }
        }
        public virtual DeviceType Type
        {
            get { return _Type; }
            set { _Type = value; }
        }
        public virtual DateTime CreationTime
        {
            get { return _CreationTime; }
            set { _CreationTime = value; }
        }

        public virtual DateTime ChangingTime
        {
            get { return _ChangingTime; }
            set { _ChangingTime = value; }
        }

        public virtual string Remark
        {
            get { return _Remark; }
            set { _Remark = value; }
        }

        protected internal virtual void ResetCtrl()
        {
            DoCtrl(ID, 0, 0, 0, 0);
        }

        protected internal virtual void DoCtrl(int ctrlCode,int param1,int param2,int param3,int param4=-1)
        {
            if (_SendBytes != null && _SendBytes.Length == 14)
            {
                _SendBytes[1] = (byte)((int)_Type==5?(int)0x80: (int)_Type);
                _SendBytes[2] = (byte)ctrlCode;
                _SendBytes[3] = (byte)param1;
                _SendBytes[4] = (byte)param2;
                _SendBytes[5] = (byte)param3;
                if (param4 != -1)
                    _SendBytes[6] = (byte)param4;
                _SendBytes[12] = Xor(_SendBytes);
            }
        }


        protected internal virtual void DoCtrl(int ctrlCode, byte[] data)
        {
            if (_SendBytes != null && _SendBytes.Length == 14)
            {
                _SendBytes[1] = (byte)((int)_Type == 5 ? (int)0x80 : (int)_Type);
                _SendBytes[2] = (byte)ctrlCode;
                _SendBytes[3] = data[0];
                _SendBytes[4] = data[1];
                _SendBytes[5] = data[2];
                _SendBytes[6] = data[3];
                _SendBytes[12] = Xor(_SendBytes);
            }
        }


        protected internal virtual byte Xor(byte[] buffer)
        {
            byte result = 0;
            for (int i = 0; i < buffer.Count() - 2; i++)
            {
                result ^= buffer[i];
            }
            return (result);
        }
    }
}
