﻿using System;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Threading;

namespace IOTransmissionAbutment
{
    public class DeviceManager
    {
        /// <summary>
        /// 做成可配置
        /// </summary>
        public static TCPServer Server;
        public static string ServerIP = "127.0.0.1";
        public static int ServerPort = 8897;

        public static List<IOTransmissionDevice> listIOTransmissionDevice = new List<IOTransmissionDevice>();

        public static ConcurrentDictionary<int, Device> dictDevice = new ConcurrentDictionary<int, Device>();
        public static List<General> listGeneral = new List<General>();
        public static ConcurrentDictionary<string, string> Generals = new ConcurrentDictionary<string, string>();

        public static Action<string> SendReport = null;
        public static Action<int, DeviceType> DeviceRemoved = null;
        public static Action<Device> DeviceUpdated = null;
        public static Action<Device> DeviceAdded = null;


        private static ConcurrentDictionary<int, DateTime> dictCarrierChargeTime = new ConcurrentDictionary<int, DateTime>();

        public static void Init(string dir)
        {
            ModelHelper.initialize(dir);
            LoadInfoFromFile();
            FillDeviceDict();
            TCPServer tcpServer = new TCPServer();
            tcpServer.WatchThread();
            InitServer(tcpServer);
            frmMain.ShowForm();
        }

        

        public static void StartSend()
        {
            new Task(() => StartSendCtrlMsgs()).Start();
        }
        internal static void InitServer(TCPServer tcpServer)
        {
            Server = tcpServer;
        }

        public static int GeneratedDeviceID()
        {
            var result = dictDevice.Keys;
            int max1 = result != null && result.Count() > 0 ? result.Max() + 1 : 1;
            return max1;
        }

        public static IOTransmissionDevice GetRollerDevice(int id)
        {
            if (dictDevice.Values != null)
            {
                Device device;
                if (dictDevice.TryGetValue(id, out device))
                {
                    IOTransmissionDevice gate = device as IOTransmissionDevice;
                    return gate;
                }
            }
            return null;
        }

       
        public static IOTransmissionDevice GetRollerDevice(string name)
        {
            if (dictDevice.Values != null)
            {
                Device device= dictDevice.Values.Any(p=>p.Name==name)? dictDevice.Values.First(p => p.Name == name):null;
                if (device!=null)
                {
                    IOTransmissionDevice gate = device as IOTransmissionDevice;
                    return gate;
                }
            }
            return null;
        }

      

        public static void DevieceIPUpdated(Device pDevice)
        {
            if (DeviceAdded != null)
            {
                DeviceAdded(pDevice);
            }
        }
        public static bool AddDeviece(Device pDevice)
        {
            bool result = dictDevice.TryAdd(pDevice.ID, pDevice);
            if (result && DeviceAdded != null)
            {
                DeviceAdded(pDevice);
            }
            return result;
        }

        public static bool RemoveDevice(int id)
        {
            Device device;
            if (dictDevice.TryRemove(id, out device))
            {
                if (DeviceRemoved != null)
                {
                    DeviceRemoved(id, device.Type);
                }
                return true;
            }
            return false;
        }

        public static void FillDeviceDict()
        {

            foreach (var item in listIOTransmissionDevice)
            {
                dictDevice[item.ID] = item;
                dictDevice[item.ID].Type = DeviceType.Other;
                item.ResetCtrl();
            }
            foreach (var item in listGeneral)
            {
                Generals[item.Name] = item.Content;
            }
            ServerIP = Generals.Values.Count >= 2 ? Generals["ServerIP"] : ServerIP;
            if (Generals.Values.Count >=2)
            {
                int temp;
                if (int.TryParse(Generals["ServerPort"], out temp))
                {
                    ServerPort = temp;
                }
            }
        }

        public static void LoadInfoFromFile()
        {
            using (var session = ModelHelper.Session)
            {
                try
                {
                    var resultGates = session.QueryOver<IOTransmissionDevice>().List();
                    listIOTransmissionDevice = resultGates.ToList();
                    var resultGenerals= session.QueryOver<General>().List();
                    listGeneral = resultGenerals.ToList();
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
            //InitElevatorFloors();
        }

        internal static void StartSendCtrlMsgs()
        {
            while (true)
            {
                System.Threading.Thread.Sleep(500);
                foreach (var item in dictDevice)
                {
                    try
                    {
                        Socket socket;
                        if (Server.dictOnlineClients.TryGetValue(item.Value.IPAddress, out socket))
                        {
                            if (!item.Value.Online)
                            {
                                CRLog.Instance.WriteLog("Online_Scan", string.Format("设备：{0}上线", item.Key));
                            }
                            item.Value.Online = true;
                            socket.Send(item.Value.SendBytes);
                            string strSended = BitConverter.ToString(item.Value.SendBytes).Replace("-", " "); ;
                            strSended = string.Format("Send to {0}-------->:{1}", item.Value.IPAddress, strSended);
                            if (SendReport != null)
                            {
                                SendReport(strSended);
                            }
                            Device device = item.Value;
                            //switch (device.Type)
                            //{
                            //    case DeviceType.PowerStation:
                            //        PowerStationDevice powerstation = device as PowerStationDevice;
                            //        if (powerstation != null && powerstation.Carrier <= 0 && powerstation.Occupied == true)
                            //        {
                            //            CLog.Instance.WriteLog("charge", string.Format("充电站占用车辆{0}，解除{1}占用", powerstation.Carrier, powerstation.Name));
                            //            powerstation.Occupied = false;
                            //        }
                            //        break;
                            //}
                        }
                        else
                        {
                            if (item.Value.Online)
                            {
                                CRLog.Instance.WriteLog("Online_Scan", string.Format("设备：{0}下线", item.Key));
                            }
                            Device device = item.Value;
                            switch (device.Type)
                            {
                                //case DeviceType.Elevator:
                                //    ElevatorDevice elevator = device as ElevatorDevice;
                                //    if (elevator != null)
                                //    {
                                //        elevator.Status = ElevatorStatus.Onkown;
                                //    }
                                //    break;
                                case DeviceType.Roller:
                                    break;
                                    //case DeviceType.PowerStation:
                                    //    PowerStationDevice powerstation = device as PowerStationDevice;
                                    //    if (powerstation != null)
                                    //    {
                                    //        //powerstation.Occupied = true;
                                    //        powerstation.PileStatus = PileStatus.Reached;
                                    //    }
                                    //    break;
                            }
                            item.Value.Online = false;
                        }
                    }
                    catch (Exception ex)
                    {

                    }
                }
            }
        }

    }
}
