﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace IOTransmissionAbutment
{
    public partial class IOTransmissionAbutmentForm : Form
    {
        IOTransmissionDevice curGate = null;
        public IOTransmissionAbutmentForm()
        {
            InitializeComponent();
            Register();

            //LocalizationTool.localizeForm(this);
            //columnHeader8.Text = LocalizationTool.getMessage("msgDevicesAbutmentmsg0064");
            //columnHeader3.Text = LocalizationTool.getMessage("msgDevicesAbutmentmsg0065");
            //columnHeader10.Text = LocalizationTool.getMessage("msgDevicesAbutmentmsg0068");
            //columnHeader1.Text = LocalizationTool.getMessage("msgDevicesAbutmentmsg0070");
            //columnHeader4.Text = LocalizationTool.getMessage("msgDevicesAbutmentmsg0069");
            //columnHeader5.Text = LocalizationTool.getMessage("msgDevicesAbutmentmsg0071");
        }
        void Register()
        {
            lvGate.SelectedIndexChanged += LvGate_SelectedIndexChanged;
            btnAdd.Click += BtnAdd_Click;
            btnDelete.Click += BtnDelete_Click;
            btnEdit.Click += BtnEdit_Click;
            this.Load += GatesForm_Load;
            TmRenew.Interval = 1000;
            TmRenew.Tick += TmRenew_Tick;
            DeviceManager.DeviceRemoved += DeviceRemoved;
            DeviceManager.DeviceAdded += DeviceAdded;
        }

       


        void DeviceAdded(Device device)
        {
            if (device != null && device.Type == DeviceType.Other)
            {
                UpdateHeaderData();
            }
        }
        void DeviceRemoved(int id, DeviceType type = DeviceType.None)
        {
            if (type == DeviceType.Other)
            {
                UpdateHeaderData();
                //InitialDateGridView();
            }
        }


        private void GatesForm_Load(object sender, EventArgs e)
        {
            LoadFromTable();
            InitialDateGridView();
        }

        void LoadFromTable(int selectedIndex = 0)
        {
            lvGate.Items.Clear();
            if (DeviceManager.dictDevice.Values != null)
            {
                var result = DeviceManager.dictDevice.Values.ToList().FindAll(p => p.Type == DeviceType.Other);
                foreach (var item in result)
                {
                    IOTransmissionDevice gate = item as IOTransmissionDevice;
                    if (gate != null)
                    {
                        ListViewItem subItem = new ListViewItem();
                        subItem.Tag = gate;
                        subItem.Text = gate.ID.ToString();
                        //subItem.SubItems.Add(item.ID.ToString());
                        subItem.SubItems.Add(gate.Name);
                        subItem.SubItems.Add(gate.IPAddress);
                        subItem.SubItems.Add(gate.CreationTime.ToString());
                        subItem.SubItems.Add(gate.ChangingTime.ToString());
                        subItem.SubItems.Add(gate.Remark);
                        lvGate.Items.Add(subItem);
                    }
                }
            }
            if (lvGate.Items.Count > selectedIndex)
            {
                lvGate.Items[selectedIndex].Selected = true;
            }
        }
        private void BtnDelete_Click(object sender, EventArgs e)
        {
            if (curGate != null)
            {
                using (var session = ModelHelper.Session)
                {
                    session.Delete(curGate);
                    session.Flush();
                }
                //DeviceManager.listGate.RemoveAll(p=>p.ID==curGate.ID);
                //Device device;
                //DeviceManager.dictDevice.TryRemove(curGate.ID, out device);
                DeviceManager.RemoveDevice(curGate.ID);
                LoadFromTable();
            }
        }

        private void BtnAdd_Click(object sender, EventArgs e)
        {
            if (CheckInput())
            {
                IOTransmissionDevice gateDevice = new IOTransmissionDevice();
                gateDevice.ID = DeviceManager.GeneratedDeviceID();
                gateDevice.Type = DeviceType.Other;
                gateDevice.Name = txtName.Text.Trim();
                gateDevice.IPAddress = txtIP.Text.Trim();
                gateDevice.CreationTime = DateTime.Now;
                gateDevice.ChangingTime = gateDevice.CreationTime;
                gateDevice.Remark = txtRemark.Text == null ? "" : txtRemark.Text;
                using (var session = ModelHelper.Session)
                {
                    session.Save(gateDevice);
                    session.Flush();
                }
                //DeviceManager.listGate.Add(gateDevice);
                //DeviceManager.dictDevice.TryAdd(gateDevice.ID, gateDevice);
                DeviceManager.AddDeviece(gateDevice);
                LoadFromTable();
            }
        }

        private void BtnEdit_Click(object sender, EventArgs e)
        {
            if (CheckInput(curGate))
            {
                if (curGate != null)
                {

                    IOTransmissionDevice gateDevice = DeviceManager.GetRollerDevice(curGate.ID);
                    if (gateDevice != null)
                    {
                        bool ipChanged = gateDevice.IPAddress != txtIP.Text.Trim();
                        gateDevice.Name = txtName.Text.Trim();
                        gateDevice.IPAddress = txtIP.Text.Trim();
                        gateDevice.ChangingTime = DateTime.Now;
                        gateDevice.Remark = txtRemark.Text == null ? "" : txtRemark.Text;
                        using (var session = ModelHelper.Session)
                        {
                            session.Update(gateDevice);
                            session.Flush();
                        }
                        if (ipChanged)
                        {
                            DeviceManager.DevieceIPUpdated(gateDevice);
                        }
                        int indexx = 0;
                        if (lvGate.SelectedIndices.Count > 0)
                        {
                            indexx = lvGate.SelectedIndices[0];
                        }
                        LoadFromTable(indexx);
                    }
                }
            }
        }

        private void LvGate_SelectedIndexChanged(object sender, EventArgs e)
        {
            curGate = null;
            if (lvGate.SelectedIndices.Count > 0)
            {
                int index = lvGate.SelectedIndices[0];
                curGate = (IOTransmissionDevice)this.lvGate.Items[index].Tag;
                RefreshCards(curGate);
            }
            RefreshCards(curGate);
        }
        private void RefreshCards(IOTransmissionDevice gate)
        {
            if (gate != null)
            {
                txtRemark.Text = gate.Remark;
                txtIP.Text = gate.IPAddress;
                txtName.Text = gate.Name;

            }
            else
            {
                txtRemark.Text = "";
                txtIP.Text = "";
                txtName.Text = "";
            }
        }

        bool CheckInput(Device device = null)
        {
            if (string.IsNullOrWhiteSpace(txtIP.Text.Trim()) || string.IsNullOrWhiteSpace(txtName.Text.Trim()))
            {
                MessageBox.Show("输入项均不能为空！");
                //MessageBox.Show(LocalizationTool.getMessage("msgDevicesAbutmentmsg0031"));
                return false;
            }
            string ip = txtIP.Text.Trim();
            string name = txtName.Text.Trim();
            IPAddress temp = null;
            if (!IPAddress.TryParse(ip, out temp))
            {
                MessageBox.Show("无效的IP地址");
                //MessageBox.Show(LocalizationTool.getMessage("msgDevicesAbutmentmsg0032"));
                return false;
            }
            bool repeated = device == null ? DeviceManager.dictDevice.Values.Any(p => p.IPAddress == ip) : DeviceManager.dictDevice.Values.Any(p => p.IPAddress == ip && p.ID != device.ID);
            if (repeated)
            {
                MessageBox.Show("设备列表中重复的IP地址！");
                //MessageBox.Show(LocalizationTool.getMessage("msgDevicesAbutmentmsg0033"));
                return false;
            }
            bool repeated2 = device == null ? DeviceManager.dictDevice.Values.Any(p => p.Name == name) : DeviceManager.dictDevice.Values.Any(p => p.Name == name && p.ID != device.ID);
            if (repeated2)
            {
                MessageBox.Show("设备列表中重复的名称！");
                //MessageBox.Show(LocalizationTool.getMessage("msgDevicesAbutmentmsg0034"));
                return false;
            }
            return true;
        }



        private StatusType Status = StatusType.线边滚筒;

        #region DateGridView
        private DataTable CurrentTable = null;
        private Dictionary<StatusType, GridViewContent> GridViewContents;

        private void InitialDateGridView()
        {
            InitialGridViewContents();

            MainDGV.RowHeadersVisible = false;
            MainDGV.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells;
            MainDGV.AllowUserToResizeColumns = false;
            MainDGV.AllowUserToResizeRows = false;
            MainDGV.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            MainDGV.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;

            UpdateHeaderData();
            UpdateMainData();
            TmRenew.Start();
        }

        private void InitialGridViewContents()
        {
            GridViewContents = new Dictionary<StatusType, GridViewContent>();

            GridViewContents[StatusType.线边滚筒] = new GridViewContentRoller();

            foreach (var content in GridViewContents.Values)
            {
                content.SetManager(MainDGV);
            }
        }

        private void UpdateHeaderData()
        {
            MainDGV.DataSource = null;
            if (CurrentTable != null)
            {
                CurrentTable.Dispose();
                CurrentTable = null;
            }
            CurrentTable = GridViewContents[Status].UpdateHeader();
            MainDGV.DataSource = CurrentTable;
        }

        private void UpdateMainData()
        {
            GridViewContents[Status].UpdateData(CurrentTable);
        }

        private void TmRenew_Tick(object sender, EventArgs e)
        {
            UpdateMainData();
        }





        class GridViewContentRoller : GridViewContent
        {
            public GridViewContentRoller()
            {
                Type = StatusType.线边滚筒;
            }

            public override void OnDispose()
            {

            }

            public override void UpdateData(DataTable table)
            {
                if (table == null) return;
                if (table.TableName != Type.ToString()) return;

                foreach (DataRow row in table.Rows)
                {
                    int index = (int)row[0];
                    IOTransmissionDevice device = DeviceManager.GetRollerDevice(index);
                    if (device != null)
                    {
                        if ((string)row[1] != device.Name) row[1] = device.Name;
                        if ((bool)row[2] != device.Online) row[2] = device.Online;
                        int outInt = device.ReceiveBytes != null ? BitConverter.ToInt16(device.ReceiveBytes, 4) : 0;
                        int inInt = device.ReceiveBytes != null ? BitConverter.ToInt16(device.ReceiveBytes, 6) : 0;
                        string outTempStr = "";
                        string intTempStr = "";
                        for (int i = 0; i < 8; i++)
                        {
                            int tempout = (outInt & 1 << i) != 0 ? 1 : 0;
                            int tempin = (inInt & 1 << i) != 0 ? 1 : 0;
                            outTempStr = outTempStr + tempout;
                            intTempStr = intTempStr + tempin;
                        }
                        //string outTempStr = device.ReceiveBytes != null ? Convert.ToString(BitConverter.ToInt16(device.ReceiveBytes, 4), 2):"";//
                        //string intTempStr = device.ReceiveBytes != null ? Convert.ToString(BitConverter.ToInt16(device.ReceiveBytes, 6), 2) : "";
                        if ((string)row[3] != outTempStr) row[3] = outTempStr;
                        if ((string)row[4] != intTempStr) row[4] = intTempStr;
                        //if ((int)row[4] != device.Carrier) row[4] = device.Carrier;
                    }

                }
            }

            public override DataTable UpdateHeader()
            {
                DataTable table = new DataTable();
                table.TableName = Type.ToString();

                table.Columns.Add("设备ID", typeof(int));
                table.Columns.Add("对接库位", typeof(string));
                table.Columns.Add("在线", typeof(bool));
                table.Columns.Add("输出控制(Y1-Y8)", typeof(string));
                table.Columns.Add("输入状态(X1-X8)", typeof(string));
                table.Columns.Add("车辆", typeof(int));
                //table.Columns.Add("设备状态", typeof(RollerStatus));
                //table.Columns.Add("车辆", typeof(int));
                //table.Columns.Add(LocalizationTool.getMessage("msgDevicesAbutmentmsg0016"), typeof(int));//"设备ID"
                //table.Columns.Add(LocalizationTool.getMessage("msgDevicesAbutmentmsg0017"), typeof(string));//"设备名称"
                //table.Columns.Add(LocalizationTool.getMessage("msgDevicesAbutmentmsg0018"), typeof(bool));//"在线"
                //table.Columns.Add(LocalizationTool.getMessage("msgDevicesAbutmentmsg0040"), typeof(RollerStatus));//"设备状态"
                //table.Columns.Add(LocalizationTool.getMessage("msgDevicesAbutmentmsg0026"), typeof(int));//"车辆"
                if (DeviceManager.dictDevice.Values != null)
                {
                    foreach (var device in DeviceManager.dictDevice.Values)
                    {
                        if (device.Type == DeviceType.Other)
                        {
                            table.Rows.Add(device.ID, device.Name, false, "", "", 0);
                        }
                    }
                }

                return table;
            }
        }
        #endregion

    }
}
