﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Pms.UserEvent.Model
{
    /// <summary>
    /// 包含AGV发送的相关数据
    /// </summary>
    public class AgvDataEventArgs : EventArgs
    {
        /// <summary>
        /// AGV编号
        /// </summary>
        public int agvID;
        /// <summary>
        /// 结束点地标号
        /// </summary>
        public int mark;
        /// <summary>
        /// 当前路线
        /// </summary>
        public int route;
        /// <summary>
        /// 无线地址
        /// </summary>
        public int address;//10 33 30 30 30 50 30 34 00 01 02 01 71 03
        /// <summary>
        /// 通信方式 1：串口   2：网络
        /// </summary>
        public int type;
        /// <summary>
        /// 叫料工位
        /// </summary>
        public int callStation;
        /// <summary>
        /// 物料ID
        /// </summary>
        public int materialID;
        /// <summary>
        /// 物料数量
        /// </summary>
        public int maNum;
        /// <summary>
        /// PADID
        /// </summary>
        public int PADID;
        /// <summary>
        /// 叫料ID
        /// </summary>
        public int callID;
    }
    /// <summary>
    /// AGV反馈回来数据
    /// </summary>
    public class AgvDateEventArgs1 : EventArgs
    {
        /// <summary>
        /// AGV编号
        /// </summary>
        public int AgvId;

        /// <summary>
        /// AGV当前的路线
        /// </summary>
        public int AGVRoute;

        /// <summary>
        /// AGV当前地标
        /// </summary>
        public int AgvMark;

        /// <summary>
        /// AGV当前的状态
        /// </summary>
        public string AgvStatus;

        /// <summary>
        /// 通讯类型
        /// </summary>
        public int Type;

    }
    public class DataEventArgs : EventArgs
    {
        public byte[] data;
        public int type;
    }

    /// <summary>
    /// AGV调度类，主要用于放行待命的AGV
    /// </summary>
    public class Dispatcher
    {
        private const int COM_FUNC = 1;
        private const int COM_ADDRESS = 3;
        private const int COM_ROUTE = 7;
        private const int COM_MARK = 8;
        private const int COM_AGV = 11;
        private const int COM_CHECK = 12;

        private const int NET_FUNC = 2;
        private const int NET_PADID = 3;
        private const int NET_MAID = 4;
        private const int NET_CALLID = 5;
        private const int NET_ROUTE = 7;
        private const int NET_MARK = 8;
        private const int NET_MANUM = 10;
        private const int NET_AGV = 11;
        private const int NET_CHECK = 12;

        private byte[] send_buffer = new byte[14] { 0x10, 0, 0, 0x30, 0, 0, 0, 0x30, 0, 0, 0, 0, 0, 0x03 };
        private byte[] send_buffer2 = new byte[10] { 0x10, 0x61, 0, 0, 0, 0, 0, 0, 0x71, 03 };
        //日志记录
        //internal static CBlackBoxFile gs_blackBoxFile;      // 黑匣子
        private static bool gs_receive0x32Flag = false;
        private SerialPortEx port;
        private UDPCommunication UdpConnect;
        /*2016-12-27 薛德龙修改 串口发送间隔时间100,net暂不变,单位毫秒，后面这个时间统一改，不再一一备注*/
        static readonly int sendSplitTime = 100;
        /*2016-12-27 薛德龙修改*/
        /// <summary>
        /// 有无日志记录功能，true ： 有 false ： 无
        /// </summary>
        public static bool writeLogFlag = false;

        /// <summary>
        /// 调度构造函数
        /// </summary>
        public Dispatcher()
        {
            CSNetComm.MatchedParity += new CSNetComm.MatchedParityHandler(MatchedParity);

            port = new SerialPortEx();
            port.MatchedParity += new SerialPortEx.MatchedParityHandler(MatchedParity);

            port.serialPort1.BaudRate = 9600;

            port.ReadingData += new EventHandler((o, e) =>
            {
                OnReadingData(e);
            });
            port.WritingData += new EventHandler((o, e) =>
            {
                OnWritingData(e);
            });
        }
        public Dispatcher(string ip, int cport)
        {
            UdpConnect = new UDPCommunication(ip, cport, 10);
            UdpConnect.MatchedParity += new UDPCommunication.MatchedParityHandler(MatchedParity);
            UdpConnect.ReadingData += new EventHandler((o, e) =>
            {
                OnReadingData(e);
            });
            UdpConnect.WritingData += new EventHandler((o, e) =>
            {
                OnWritingData(e);
            });
        }

        #region 后来增加事件，用于通知是否在操作串口
        /// <summary>
        /// 串口写
        /// </summary>
        public event EventHandler WritingData;
        /// <summary>
        /// 串口读
        /// </summary>
        public event EventHandler ReadingData;
        void OnReadingData(EventArgs e)
        {
            if (ReadingData != null)
            {
                ReadingData(this, e);
            }
        }
        void OnWritingData(EventArgs e)
        {
            if (WritingData != null)
            {
                WritingData(this, e);
            }
        }
        /// <summary>
        /// 串口是否打开
        /// </summary>
        /// <returns></returns>
        public bool ComIsOpen()
        {
            if (port != null)
            {
                return port.IsOpen();
            }
            return false;
        }
        #endregion

        #region 叫料相关事件
        public delegate void WorkLineCalledHandler(object sender, AgvDataEventArgs e);
        /// <summary>
        /// 产线叫料
        /// </summary>
        public event WorkLineCalledHandler WorkLineCalled;
        private void OnWorkLineCalled(AgvDataEventArgs e)
        {
            if (WorkLineCalled != null)
            {
                WorkLineCalled(this, e);
            }
        }
        public delegate void WorklineCancledHandler(object sender, AgvDataEventArgs e);
        /// <summary>
        /// 叫料点取消叫料
        /// </summary>
        public event WorklineCancledHandler WorkLineCancled;
        private void OnWorkLineCancled(AgvDataEventArgs e)
        {
            if (WorkLineCancled != null)
            {
                WorkLineCancled(this, e);
            }
        }

        public delegate void WorkLineReplyedHandler(object sender, AgvDataEventArgs e);
        /// <summary>
        /// 叫料点回应调度的放车请求
        /// </summary>
        public event WorkLineReplyedHandler WorkLineReplyed;
        private void OnWorkLineReplyed(AgvDataEventArgs e)
        {
            if (WorkLineReplyed != null)
            {
                WorkLineReplyed(this, e);
            }
        }
        public delegate void AskAgvWorkStatusHandler(object sender, AgvDateEventArgs1 e);
        /// <summary>
        /// 轮询AGV反馈回来的事件
        /// </summary>
        public event AskAgvWorkStatusHandler AskAgvWorkStatus;

        private void OnAskAgvWorkStatus(AgvDateEventArgs1 e)
        {
            if (AskAgvWorkStatus != null)
            {
                AskAgvWorkStatus(this, e);
            }
        }
        #endregion

        #region 调度相关事件
        /// <summary>
        /// 处理AgvReadEnding事件的方法
        /// </summary>
        /// <param name="sender">事件源</param>
        /// <param name="e">包含事件数据的 Dispatch.AgvDataEventArgs</param>
        public delegate void AgvReadEndingHandler(object sender, AgvDataEventArgs e);

        /// <summary>
        /// 当结束点有AGV时发生
        /// </summary>
        public event AgvReadEndingHandler AgvReadEnding;
        private void OnAgvReadEnding(object sender, AgvDataEventArgs e)
        {
            if (AgvReadEnding != null)
            {
                AgvReadEnding(sender, e);
            }
        }

        /// <summary>
        /// 处理AgvReadEnding事件的方法
        /// </summary>
        /// <param name="sender">事件源</param>
        /// <param name="e">包含事件数据的 Dispatch.AgvDataEventArgs</param>
        public delegate void AgvReadStopingHandler(object sender, AgvDataEventArgs e);

        /// <summary>
        /// 当停止点有AGV时发生
        /// </summary>
        public event AgvReadStopingHandler AgvReadStoping;
        private void OnAgvReadStoping(object sender, AgvDataEventArgs e)
        {
            if (AgvReadStoping != null)
            {
                AgvReadStoping(sender, e);
            }
        }

        /// <summary>
        /// 处理AgvReleased事件的方法
        /// </summary>
        /// <param name="sender">事件源</param>
        /// <param name="e">包含事件数据的 Dispatch.AgvDataEventArgs</param>
        public delegate void AgvReleasedHandler(object sender, AgvDataEventArgs e);

        /// <summary>
        /// 当结束点AGV放行成功时发生
        /// </summary>
        public event AgvReleasedHandler AgvReleased;
        private void OnAgvReleased(object sender, AgvDataEventArgs e)
        {
            if (AgvReleased != null)
            {
                AgvReleased(sender, e);
            }
        }

        /// <summary>
        /// 处理AgvReleaseOnStop事件的方法
        /// </summary>
        /// <param name="sender">事件源</param>
        /// <param name="e">包含事件数据的 Dispatch.AgvDataEventArgs</param>
        public delegate void AgvReleaseOnStopHandler(object sender, AgvDataEventArgs e);
        /// <summary>
        /// 当AGV停止点放行成功时发生
        /// </summary>
        public event AgvReleaseOnStopHandler AgvReleasedOnStop;
        private void OnAgvReleasedOnStop(object sender, AgvDataEventArgs e)
        {
            if (AgvReleasedOnStop != null)
            {
                AgvReleasedOnStop(sender, e);
            }
        }
        #endregion

        #region 调度相关函数
        /// <summary>
        /// 用于发车成功后，通知叫料盒
        /// </summary>
        /// <param name="route"></param>
        /// <param name="address"></param>
        /// <param name="mark"></param>
        /// <param name="agv"></param>
        public void Send45ToWorkLine(int route, int address = 0, int mark = 0, int agv = 0)
        {
            byte[] buffer = new byte[14] { 0x10, 0, 0x30, 0x30, 0x30, 0x50, 0x30, 0x30, 0, 0, 0, 0, 0, 0x03 };
            buffer[COM_FUNC] = 0x45;
            buffer[COM_ADDRESS] = (byte)(address + 0x30);
            /*2016-12-27 薛德龙修改*/
            int Route = 0x3030 + route;
            buffer[COM_ROUTE - 1] = (byte)((Route >> 8) & 0xFF);
            buffer[COM_ROUTE] = (byte)(Route & 0xFF);
            /*2016-12-27 薛德龙修改*/
            buffer[COM_MARK] = (byte)((mark & 0xff00) >> 8);
            buffer[COM_MARK + 1] = (byte)(mark & 0x00ff);
            buffer[COM_AGV] = (byte)agv;
            buffer[COM_CHECK] = SerialPortEx.XOR_Check(buffer, 12);
            //for (int i = 0; i < 3; i++)
            //{
            UdpConnect.SendData(buffer, null);
            //port.Write(buffer, 0, buffer.Length);
            Thread.Sleep(sendSplitTime);
            //}
        }
        /// <summary>
        /// 通知网络叫料端，已派车
        /// </summary>
        /// <param name="route">路线</param>
        /// <param name="terminalID">叫料终端</param>
        /// <param name="callID">叫料ID</param>
        /// <param name="agvID">AGVID</param>
        public void Send42ToWorkLineNet(int route, int terminalID, int callID, int agvID)
        {
            byte[] buffer = new byte[14] { 0x10, 0x0E, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03 };
            buffer[NET_FUNC] = 0x52;
            buffer[NET_PADID] = (byte)(terminalID);
            buffer[NET_CALLID] = (byte)(callID);
            buffer[NET_ROUTE] = (byte)(route);
            buffer[NET_AGV] = (byte)agvID;
            buffer[12] = SerialPortEx.XOR_Check(buffer, 12);

            for (int i = 0; i < 3; i++)
            {
                CSNetComm.SendBytes(0, buffer, buffer.Length);
                Thread.Sleep(200);
            }
        }
        /// <summary>
        /// 发送协议开启报警灯
        /// </summary>
        /// <param name="ID"></param>
        public void SendStartBeepData(int ID)
        {
            byte[] data = new byte[16] { 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
            data[0] = 0x10;
            data[4] = 0x01;
            data[5] = 0x99;
            data[6] = 0x99;
            data[7] = 0x99;
            data[8] = 0x99;
            data[14] = (byte)ID;
            data[15] = SerialPortEx.XOR_Check(data, 15);
            //for (int i = 0; i < 5; i++)
            //{
            UdpConnect.SendData(data, null);
            //}
        }
        /// <summary>
        /// 发送协议关闭报警灯
        /// </summary>
        /// <param name="ID"></param>
        public void SendStopBeepData(int ID)
        {

            byte[] data = new byte[16];
            data[0] = 0x10;
            data[4] = 0x00;

            data[5] = 0x99;
            data[6] = 0x99;
            data[7] = 0x99;
            data[8] = 0x99;

            data[14] = (Byte)ID;
            data[15] = SerialPortEx.XOR_Check(data, 15);
            //for (int i = 0; i < 5; i++)
            //{
            UdpConnect.SendData(data, null);
            //}
        }
        /// <summary>
        /// 通知网络叫料端，AGV已到目的地
        /// </summary>
        /// <param name="route">路线</param>
        /// <param name="terminalID">叫料终端</param>
        /// <param name="callID">叫料ID</param>
        /// <param name="agvID">AGVID</param>
        public void Send45ToWorkLineNet(int route, int terminalID, int callID, int agvID)
        {
            byte[] buffer = new byte[14] { 0x10, 0x0E, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03 };
            buffer[NET_FUNC] = 0x55;
            buffer[NET_PADID] = (byte)(terminalID);
            buffer[NET_CALLID] = (byte)(callID);
            buffer[NET_ROUTE] = (byte)(route);
            buffer[NET_AGV] = (byte)agvID;
            buffer[12] = SerialPortEx.XOR_Check(buffer, 12);

            for (int i = 0; i < 3; i++)
            {
                CSNetComm.SendBytes(0, buffer, buffer.Length);
                Thread.Sleep(200);
            }
        }
        /// <summary>
        /// 开始串口监听，执行后才能进行数据收发
        /// </summary>
        /// <param name="portName">监听的串口号</param>
        public void StartListen2(string portName, int baudrate1)
        {
            if (port != null && !port.serialPort2.IsOpen)
            {
                if (!port.serialPort2.IsOpen)
                {
                    try
                    {
                        port.serialPort2.PortName = portName;
                        port.serialPort2.BaudRate = baudrate1;
                        port.Open1();
                    }
                    catch (System.Exception ex)
                    {
                        string logInfo = ex.Message;
                        StackTrace st = new StackTrace(true);
                        if (st.FrameCount > 1)
                        {
                            StackFrame sf = st.GetFrame(1);
                            logInfo += sf.GetFileName();
                            logInfo += sf.GetMethod().Name;
                            logInfo += sf.GetFileLineNumber();
                        }
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, "7:" + logInfo);
                        throw new Exception(logInfo);
                    }
                }
            }
        }
        /// <summary>
        /// 开始串口监听，执行后才能进行数据收发
        /// </summary>
        /// <param name="portName">监听的串口号</param>
        public void StartListen(string portName)
        {
            if (port != null && !port.serialPort1.IsOpen)
            {
                if (!port.serialPort1.IsOpen)
                {
                    try
                    {
                        port.serialPort1.PortName = portName;
                        port.Open();
                    }
                    catch (System.Exception ex)
                    {
                        string logInfo = ex.Message;
                        StackTrace st = new StackTrace(true);
                        if (st.FrameCount > 1)
                        {
                            StackFrame sf = st.GetFrame(1);
                            logInfo += sf.GetFileName();
                            logInfo += sf.GetMethod().Name;
                            logInfo += sf.GetFileLineNumber();
                        }
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, "7:" + logInfo);
                        throw new Exception(logInfo);
                    }
                }
            }
        }
        /// <summary>
        /// 开始网络监听
        /// </summary>
        /// <param name="ip">ip地址</param>
        /// <param name="port">端口</param>
        public void StartListenNet(string ip, int port)
        {
            CSNetComm.Start(ip, port);
        }
        /// <summary>
        /// 开始监听，执行后才能进行数据收发
        /// </summary>
        /// <param name="portName">监听的串口号</param>
        public void StartListenUDP()
        {
            UdpConnect.Open();
        }
        /// <summary>
        /// 关闭监听，关闭后将不能进行数据收发
        /// </summary>
        public void StopListen()
        {
            if (port != null && port.serialPort1.IsOpen)
            {
                try
                {
                    port.Close();
                }
                catch (System.Exception ex)
                {
                    string logInfo = ex.Message;
                    StackTrace st = new StackTrace(true);
                    if (st.FrameCount > 1)
                    {
                        StackFrame sf = st.GetFrame(1);
                        logInfo += sf.GetFileName();
                        logInfo += sf.GetMethod().Name;
                        logInfo += sf.GetFileLineNumber();
                    }
                    //日志记录
                    //gs_blackBoxFile.WriteFile(100, "6:" + logInfo);
                    throw new Exception("关闭串口：" + logInfo);
                }

            }
        }
        /// <summary>
        /// 发送数据10 61询问
        /// </summary>
        public void SendAgvData61()
        {
            if (port != null)
            {
                port.Write1(send_buffer2, 0, send_buffer2.Length);
            }
        }
        /// <summary>
        /// 发送数据10 61询问
        /// </summary>
        public void SendAgvData61Net()
        {
            CSNetComm.SendBytes(0, send_buffer2, send_buffer2.Length);
        }
        /// <summary>
        /// 发送数据10 61询问
        /// </summary>
        public void SendAgvData61Udp()
        {
            if (UdpConnect != null)
            {
                UdpConnect.SendData(send_buffer2, null);
            }
        }
        /// <summary>
        /// 待命点放行AGV函数，只有在结束点处有AGV时，放行AGV才能生效
        /// </summary>
        /// <param name="route">放行路线</param>
        /// <param name="address">AGV无线地址，默认0</param>
        /// <param name="mark">放行地标号</param>
        /// <param name="agv">放行AGV编号，默认0</param>
        /// <param name="low">低八位，默认0；注哪个工位要停，对应哪位为1</param>
        /// <param name="high">高八位，默认0</param>
        public void ReleaseAgv(int route, int mark, byte low = 0, byte high = 0, int address = 0, int agv = 0)
        {
            byte[] buffer = new byte[14] { 0x10, 0, 0x30, 0x30, 0x30, 0x50, 0x30, 0x30, 0, 0, 0, 0, 0, 0x03 };
            if (low != 0 || high != 0)
            {
                buffer[4] = high;
                buffer[5] = low;
            }
            buffer[COM_FUNC] = 0x31;
            buffer[COM_ADDRESS] = (byte)(address + 0x30);
            /*2016-12-27 薛德龙修改*/
            int Route = 0x3030 + route;
            buffer[COM_ROUTE - 1] = (byte)((Route >> 8) & 0xFF);
            buffer[COM_ROUTE] = (byte)(Route & 0xFF);
            /*2016-12-27 薛德龙修改*/
            buffer[COM_MARK] = (byte)((mark & 0xff00) >> 8);
            buffer[COM_MARK + 1] = (byte)(mark & 0x00ff);
            buffer[COM_AGV] = (byte)agv;
            buffer[COM_CHECK] = SerialPortEx.XOR_Check(buffer, 12);

            gs_receive0x32Flag = true;
            for (int i = 0; i < 5 && gs_receive0x32Flag; i++)
            {
                //port.Write(buffer, 0, buffer.Length);
                UdpConnect.SendData(buffer, null);
                Thread.Sleep(sendSplitTime);

            }
            if (writeLogFlag)
            {
                #region 写日志
                //string str = "结束点放行AGV（ReleaseAgv）， 结束点地标： " + mark.ToString() + "路线： " + route.ToString() + "\n";
                string str = "";
                for (int i = 0; i < buffer.Length; i++)
                    str += Convert.ToString(buffer[i], 16).ToString() + " ";
                //日志记录
                //gs_blackBoxFile.WriteFile(100, str);
                #endregion
            }
        }

        /// <summary>
        /// 停止点放行AGV函数
        /// </summary>
        /// <param name="mark">放行地标号</param>
        ///   <param name="mark">放行路线</param>
        /// <param name="address">AGV无线地址，默认0</param>
        /// <param name="agv">放行AGV编号，默认0</param>
        public void ReleaseAgvOnStop(int mark, int route, int address = 0, int agv = 0)
        {
            byte[] buffer = new byte[14] { 0x10, 0, 0x30, 0x30, 0x30, 0x50, 0x30, 0x30, 0, 0, 0, 0, 0, 0x03 };
            buffer[COM_FUNC] = 0x37;
            buffer[COM_ADDRESS] = (byte)(address + 0x30);
            /*2016-12-27 薛德龙修改*/
            int Route = 0x3030 + route;
            buffer[COM_ROUTE - 1] = (byte)((Route >> 8) & 0xFF);
            buffer[COM_ROUTE] = (byte)(Route & 0xFF);
            /*2016-12-27 薛德龙修改*/
            buffer[COM_MARK] = (byte)((mark & 0xff00) >> 8);
            buffer[COM_MARK + 1] = (byte)(mark & 0x00ff);
            buffer[COM_AGV] = (byte)agv;
            buffer[COM_CHECK] = SerialPortEx.XOR_Check(buffer, 12);
            for (int i = 0; i < 5; i++)
            {
                UdpConnect.SendData(buffer, null);
                //port.Write(buffer, 0, buffer.Length);
                Thread.Sleep(sendSplitTime);
            }
            if (writeLogFlag)
            {
                #region 写日志
                string str = "停止点放行AGV（ReleaseAgvOnStop）， 停止点地标： " + mark.ToString() + "\n";
                for (int i = 0; i < buffer.Length; i++)
                    str += Convert.ToString(buffer[i], 16).ToString() + " ";
                //日志记录
                //gs_blackBoxFile.WriteFile(100, str);
                #endregion
            }
        }
        /// <summary>
        /// 数据解析
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void MatchedParity(object sender, DataEventArgs e)
        {
            byte[] buffer;
            if (e.type == 1)
            {
                buffer = e.data;
                buffer.CopyTo(send_buffer, 0);
                if (buffer[COM_FUNC] == 0x33)// && buffer[COM_ROUTE] > 0x30
                {
                    if (port.serialPort1.IsOpen)
                    {
                        //回复AGV
                        send_buffer[COM_FUNC] = 0x34;
                        send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                        port.Write(send_buffer, 0, send_buffer.Length);
                        //通知订阅了AGV到结束点事件处理器
                        AgvDataEventArgs e1 = new AgvDataEventArgs();
                        e1.agvID = buffer[COM_AGV];
                        e1.address = buffer[COM_ADDRESS] - 0x30;
                        /*2016-12-27 薛德龙修改*/
                        e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                        /*2016-12-27 薛德龙修改*/
                        e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                        e1.type = e.type;
                        OnAgvReadEnding(this, e1);
                        if (writeLogFlag)
                        {
                            #region 写日志
                            string str = "";
                            for (int i = 0; i < buffer.Length; i++)
                                str += Convert.ToString(buffer[i], 16).ToString() + " ";
                            //日志记录
                            //gs_blackBoxFile.WriteFile(100, str);
                            #endregion
                        }
                    }
                }
                else if (buffer[COM_FUNC] == 0x32)
                {
                    gs_receive0x32Flag = false;
                    //通知订阅了AGV放行成功事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReleased(this, e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x35)
                {
                    send_buffer[COM_FUNC] = 0x36;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    port.Write(send_buffer, 0, send_buffer.Length);
                    //通知订阅了AGV到结束点事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReadStoping(this, e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x38)
                {
                    //通知订阅了AGV放行成功事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReleasedOnStop(this, e1);

                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x41)
                {
                    //回复AGV
                    send_buffer[COM_FUNC] = 0x43;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    port.Write(send_buffer, 0, send_buffer.Length);
                    //通知订阅了产线叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineCalled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x42)
                {
                    //回复AGV
                    send_buffer[COM_FUNC] = 0x44;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    port.Write(send_buffer, 0, send_buffer.Length);
                    //通知订阅了产线取消叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineCancled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x46)
                {
                    //通知订阅了产线回复45事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x62)//
                {

                    AgvDateEventArgs1 e1 = new AgvDateEventArgs1();
                    int bMarkNum = 0;
                    bMarkNum = buffer[3] | ((buffer[7] & 0xC0) << 2);
                    e1.AgvId = buffer[2];
                    if (bMarkNum > 0)
                    {
                        e1.AgvMark = bMarkNum;
                    }
                    e1.Type = e.type;
                    e1.AGVRoute = buffer[6];
                    {
                        switch ((byte)(buffer[5] & 0x7f))
                        {
                            case 0x01:
                                e1.AgvStatus = "急停";
                                break;
                            case 0x02:
                                e1.AgvStatus = "防撞机构err";
                                break;
                            case 0x03:
                                e1.AgvStatus = "牵引棒";
                                break;
                            case 0x04:
                                e1.AgvStatus = "障碍物";
                                break;
                            case 0x05:
                                e1.AgvStatus = "驱动报警";
                                break;
                            case 0x06:
                                e1.AgvStatus = "后驱动异常";
                                break;
                            case 0x07:
                                e1.AgvStatus = "脱轨";
                                break;
                            case 0x08:
                                e1.AgvStatus = "低电压报警";
                                break;
                            case 0x09:
                                e1.AgvStatus = "低电压停机";
                                break;
                            case 0x0A:
                                e1.AgvStatus = "超载";
                                break;
                            case 0x0B:
                                e1.AgvStatus = "驱动提升";
                                break;
                            case 0x0C:
                                e1.AgvStatus = "后牵引";
                                break;
                            case 0x0D:
                                e1.AgvStatus = "";
                                break;
                            case 0x0E:
                                e1.AgvStatus = "启动";
                                break;
                            case 0x0F:
                                e1.AgvStatus = "复位";
                                break;
                            case 0x10:
                                e1.AgvStatus = "方向切换";
                                break;
                            case 0x40:
                                e1.AgvStatus = "运行";
                                break;
                            case 0x41:
                                e1.AgvStatus = "停止";
                                break;
                            case 0x43:
                                e1.AgvStatus = "停止点";
                                break;
                            case 0x42:
                                e1.AgvStatus = "结束点";
                                break;
                            case 0x44:
                                e1.AgvStatus = "充电中";
                                break;
                        }
                    }
                    OnAskAgvWorkStatus(e1);
                    //OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
            }
            else if (e.type == 2)
            {
                buffer = e.data;
                buffer.CopyTo(send_buffer, 0);
                if (buffer[NET_FUNC] == 0x41)
                {
                    //回复AGV
                    send_buffer[NET_FUNC] = 0x43;
                    send_buffer[NET_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    for (int i = 0; i < 3; i++)
                    {
                        CSNetComm.SendBytes(0, send_buffer, send_buffer.Length);
                        Thread.Sleep(200);
                    }
                    //port.Write(send_buffer, 0, send_buffer.Length);
                    //通知订阅了产线叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[NET_AGV];
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[NET_ROUTE - 1] << 8) & 0xFF) + buffer[NET_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    //  e1.route = buffer[NET_ROUTE];
                    e1.mark = ((int)buffer[NET_MARK] << 8 | buffer[NET_MARK + 1]);
                    e1.type = e.type;
                    e1.callStation = buffer[NET_MARK + 1];
                    e1.materialID = buffer[NET_MAID];
                    e1.maNum = buffer[NET_MANUM];
                    e1.PADID = buffer[NET_PADID];
                    e1.callID = buffer[NET_CALLID];
                    OnWorkLineCalled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[NET_FUNC] == 0x62)//罗国2017-7-20
                {
                    AgvDateEventArgs1 e1 = new AgvDateEventArgs1();
                    int bMarkNum = 0;
                    bMarkNum = buffer[3] | ((buffer[7] & 0xC0) << 2);
                    e1.AgvId = buffer[2];
                    if (bMarkNum > 0)
                    {
                        e1.AgvMark = bMarkNum;
                    }
                    e1.Type = e.type;
                    e1.AGVRoute = buffer[6];
                    {
                        switch ((byte)(buffer[5] & 0x7f))
                        {
                            case 0x01:
                                e1.AgvStatus = "急停";
                                break;
                            case 0x02:
                                e1.AgvStatus = "防撞机构err";
                                break;
                            case 0x03:
                                e1.AgvStatus = "牵引棒";
                                break;
                            case 0x04:
                                e1.AgvStatus = "障碍物";
                                break;
                            case 0x05:
                                e1.AgvStatus = "驱动报警";
                                break;
                            case 0x06:
                                e1.AgvStatus = "后驱动异常";
                                break;
                            case 0x07:
                                e1.AgvStatus = "脱轨";
                                break;
                            case 0x08:
                                e1.AgvStatus = "低电压报警";
                                break;
                            case 0x09:
                                e1.AgvStatus = "低电压停机";
                                break;
                            case 0x0A:
                                e1.AgvStatus = "超载";
                                break;
                            case 0x0B:
                                e1.AgvStatus = "驱动提升";
                                break;
                            case 0x0C:
                                e1.AgvStatus = "后牵引";
                                break;
                            case 0x0D:
                                e1.AgvStatus = "";
                                break;
                            case 0x0E:
                                e1.AgvStatus = "启动";
                                break;
                            case 0x0F:
                                e1.AgvStatus = "复位";
                                break;
                            case 0x10:
                                e1.AgvStatus = "方向切换";
                                break;
                            case 0x40:
                                e1.AgvStatus = "运行";
                                break;
                            case 0x41:
                                e1.AgvStatus = "停止";
                                break;
                            case 0x43:
                                e1.AgvStatus = "停止点";
                                break;
                            case 0x42:
                                e1.AgvStatus = "结束点";
                                break;
                            case 0x44:
                                e1.AgvStatus = "充电中";
                                break;
                        }
                    }
                    OnAskAgvWorkStatus(e1);
                    //OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[NET_FUNC] == 0x42)
                {
                    //回复AGV
                    send_buffer[NET_FUNC] = 0x44;
                    send_buffer[NET_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    for (int i = 0; i < 3; i++)
                    {
                        CSNetComm.SendBytes(0, send_buffer, send_buffer.Length);
                        Thread.Sleep(200);
                    }

                    //通知订阅了产线取消叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[NET_AGV];
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[NET_ROUTE - 1] << 8) & 0xFF) + buffer[NET_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    //  e1.route = buffer[NET_ROUTE];
                    e1.mark = ((int)buffer[NET_MARK] << 8 | buffer[NET_MARK + 1]);
                    e1.type = e.type;
                    e1.callStation = buffer[NET_MARK + 1];
                    e1.materialID = buffer[NET_MAID];
                    e1.maNum = buffer[NET_MANUM];
                    e1.PADID = buffer[NET_PADID];
                    e1.callID = buffer[NET_CALLID];
                    OnWorkLineCancled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[NET_FUNC] == 0x46)
                {
                    //通知订阅了产线回复45事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[NET_AGV];
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[NET_ROUTE - 1] << 8) & 0xFF) + buffer[NET_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    //  e1.route = buffer[NET_ROUTE];
                    e1.mark = ((int)buffer[NET_MARK] << 8 | buffer[NET_MARK + 1]);
                    e1.type = e.type;
                    e1.callStation = buffer[NET_MARK + 1];
                    e1.materialID = buffer[NET_MAID];
                    e1.maNum = buffer[NET_MANUM];
                    e1.PADID = buffer[NET_PADID];
                    e1.callID = buffer[NET_CALLID];
                    OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
            }
            else if (e.type == 3)
            {
                buffer = e.data;
                buffer.CopyTo(send_buffer, 0);
                if (buffer[COM_FUNC] == 0x33)
                {
                    if (UdpConnect != null)
                    {
                        //回复AGV
                        send_buffer[COM_FUNC] = 0x34;
                        send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                        UdpConnect.SendData(send_buffer, null);

                        //通知订阅了AGV到结束点事件处理器
                        AgvDataEventArgs e1 = new AgvDataEventArgs();
                        e1.agvID = buffer[COM_AGV];
                        e1.address = buffer[COM_ADDRESS] - 0x30;
                        /*2016-12-27 薛德龙修改*/
                        e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                        /*2016-12-27 薛德龙修改*/
                        e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                        e1.type = e.type;
                        OnAgvReadEnding(this, e1);
                        if (writeLogFlag)
                        {
                            #region 写日志
                            string str = "";
                            for (int i = 0; i < buffer.Length; i++)
                                str += Convert.ToString(buffer[i], 16).ToString() + " ";
                            //日志记录
                            //gs_blackBoxFile.WriteFile(100, str);
                            #endregion
                        }
                    }
                }
                else if (buffer[COM_FUNC] == 0x32)
                {
                    gs_receive0x32Flag = false;
                    //通知订阅了AGV放行成功事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReleased(this, e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x35)
                {
                    UdpConnect.SendData(send_buffer, null);
                    send_buffer[COM_FUNC] = 0x36;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    UdpConnect.SendData(send_buffer, null);

                    //通知订阅了AGV到结束点事件处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReadStoping(this, e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x38)
                {
                    //通知订阅了AGV放行成功事件处理器
                    UdpConnect.SendData(send_buffer, null);
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnAgvReleasedOnStop(this, e1);

                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x41)
                {
                    //回复AGV
                    send_buffer[COM_FUNC] = 0x43;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    //for (int i = 0; i < 3; i++)
                    {
                        UdpConnect.SendData(send_buffer, null);
                    }
                    //通知订阅了产线叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineCalled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x42)
                {
                    //回复AGV
                    send_buffer[COM_FUNC] = 0x44;
                    send_buffer[COM_CHECK] = SerialPortEx.XOR_Check(send_buffer, 12);
                    UdpConnect.SendData(send_buffer, null);

                    //通知订阅了产线取消叫料事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineCancled(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x46)
                {
                    //通知订阅了产线回复45事件的处理器
                    AgvDataEventArgs e1 = new AgvDataEventArgs();
                    e1.agvID = buffer[COM_AGV];
                    e1.address = buffer[COM_ADDRESS] - 0x30;
                    /*2016-12-27 薛德龙修改*/
                    e1.route = ((buffer[COM_ROUTE - 1] << 8)) + buffer[COM_ROUTE] - 0x3030;
                    /*2016-12-27 薛德龙修改*/
                    e1.mark = ((int)buffer[COM_MARK] << 8 | buffer[COM_MARK + 1]);
                    e1.type = e.type;
                    OnWorkLineReplyed(e1);
                    if (writeLogFlag)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 16).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(100, str);
                        #endregion
                    }
                }
                else if (buffer[COM_FUNC] == 0x62)//
                {
                    AgvDateEventArgs1 e1 = new AgvDateEventArgs1();
                    int bMarkNum = 0;
                    bMarkNum = buffer[3] | ((buffer[7] & 0xC0) << 2);
                    e1.AgvId = buffer[2];
                    if (bMarkNum > 0)
                    {
                        e1.AgvMark = bMarkNum;
                    }
                    e1.Type = e.type;
                    e1.AGVRoute = buffer[6];
                    {
                        switch ((byte)(buffer[5] & 0x7f))
                        {
                            case 0x01:
                                e1.AgvStatus = "急停";
                                break;
                            case 0x02:
                                e1.AgvStatus = "防撞机构err";
                                break;
                            case 0x03:
                                e1.AgvStatus = "牵引棒";
                                break;
                            case 0x04:
                                e1.AgvStatus = "障碍物";
                                break;
                            case 0x05:
                                e1.AgvStatus = "驱动报警";
                                break;
                            case 0x06:
                                e1.AgvStatus = "后驱动异常";
                                break;
                            case 0x07:
                                e1.AgvStatus = "脱轨";
                                break;
                            case 0x08:
                                e1.AgvStatus = "低电压报警";
                                break;
                            case 0x09:
                                e1.AgvStatus = "低电压停机";
                                break;
                            case 0x0A:
                                e1.AgvStatus = "超载";
                                break;
                            case 0x0B:
                                e1.AgvStatus = "驱动提升";
                                break;
                            case 0x0C:
                                e1.AgvStatus = "后牵引";
                                break;
                            case 0x0D:
                                e1.AgvStatus = "";
                                break;
                            case 0x0E:
                                e1.AgvStatus = "启动";
                                break;
                            case 0x0F:
                                e1.AgvStatus = "复位";
                                break;
                            case 0x10:
                                e1.AgvStatus = "方向切换";
                                break;
                            case 0x40:
                                e1.AgvStatus = "运行";
                                break;
                            case 0x41:
                                e1.AgvStatus = "停止";
                                break;
                            case 0x43:
                                e1.AgvStatus = "停止点";
                                break;
                            case 0x42:
                                e1.AgvStatus = "结束点";
                                break;
                            case 0x44:
                                e1.AgvStatus = "充电中";
                                break;
                        }
                    }
                    OnAskAgvWorkStatus(e1);
                    //OnWorkLineReplyed(e1);
                    if (true)
                    {
                        #region 写日志
                        string str = "";
                        for (int i = 0; i < buffer.Length; i++)
                            str += Convert.ToString(buffer[i], 10).ToString() + " ";
                        //日志记录
                        //gs_blackBoxFile.WriteFile(1001, str);
                        #endregion
                    }
                }
            }
        }//end of MatchedParity
        #endregion
    }
}
