﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Pms.UserEvent.Model
{
    internal class SerialPortEx
    {
        public SerialPortEx()
        {
            serialPort1 = new SerialPort();
            serialPort1.PortName = "COM1";
            serialPort1.BaudRate = 9600;
            serialPort1.StopBits = StopBits.One;
            serialPort1.Parity = Parity.None;
            serialPort1.DataBits = 8;
            serialPort1.ReceivedBytesThreshold = 1;
            serialPort1.ReadTimeout = 1000;
            serialPort1.DataReceived += new SerialDataReceivedEventHandler(serialPort1_DataReceived);

            serialPort2 = new SerialPort();
            serialPort2.PortName = "COM1";
            serialPort2.BaudRate = 9600;
            serialPort2.StopBits = StopBits.One;
            serialPort2.Parity = Parity.None;
            serialPort2.DataBits = 8;
            serialPort2.ReceivedBytesThreshold = 1;
            serialPort2.ReadTimeout = 1000;
            serialPort2.DataReceived += new SerialDataReceivedEventHandler(serialPort2_DataReceived);

        }
        /************************************************************************/
        /*添加了读写串口事件*/
        /************************************************************************/
        public event EventHandler ReadingData;
        public event EventHandler WritingData;
        void OnReadingData(EventArgs e)
        {
            if (ReadingData != null)
            {
                ReadingData(this, e);
            }
        }
        void OnWritingData(EventArgs e)
        {
            if (WritingData != null)
            {
                WritingData(this, e);
            }
        }
        internal bool IsOpen()
        {
            return serialPort1.IsOpen;
        }
        public void Write(string text)
        {
            try
            {
                if (serialPort1.IsOpen)
                {
                    serialPort1.Write(text);
                }
                OnWritingData(null);
            }
            catch (System.Exception ex)
            {
                string logInfo = ex.Message;
                StackTrace st = new StackTrace(true);
                if (st.FrameCount > 1)
                {
                    StackFrame sf = st.GetFrame(1);
                    logInfo += sf.GetFileName();
                    logInfo += sf.GetMethod().Name;
                    logInfo += sf.GetFileLineNumber();
                }
                //日志记录
                //Dispatcher.gs_blackBoxFile.WriteFile(100, "5:" + logInfo);
                throw new Exception("写串口:" + logInfo);
            }
        }
        public void Write(byte[] buffer, int offset, int count)
        {
            try
            {
                if (Dispatcher.writeLogFlag)
                {
                    #region 写日志
                    string str = "";
                    for (int i = 0; i < buffer.Length; i++)
                        str += Convert.ToString(buffer[i], 16).ToString() + " ";
                    //日志记录
                    //Dispatcher.gs_blackBoxFile.WriteFile(100, str);
                    #endregion
                }
                if (serialPort1.IsOpen)
                {
                    serialPort1.Write(buffer, offset, count);
                }
                OnWritingData(null);
            }
            catch (System.Exception ex)
            {
                string logInfo = ex.Message;
                StackTrace st = new StackTrace(true);
                if (st.FrameCount > 1)
                {
                    StackFrame sf = st.GetFrame(1);
                    logInfo += sf.GetFileName();
                    logInfo += sf.GetMethod().Name;
                    logInfo += sf.GetFileLineNumber();
                }
                //日志记录
                //Dispatcher.gs_blackBoxFile.WriteFile(100, "4:" + logInfo);
                throw new Exception("写串口:" + logInfo);
            }
        }

        public void Write1(byte[] buffer, int offset, int count)
        {
            try
            {

                if (Dispatcher.writeLogFlag)
                {
                    #region 写日志
                    string str = "";
                    for (int i = 0; i < buffer.Length; i++)
                        str += Convert.ToString(buffer[i], 16).ToString() + " ";
                    //日志记录
                    //Dispatcher.gs_blackBoxFile.WriteFile(100, str);
                    #endregion
                }
                if (serialPort2.IsOpen)
                {
                    serialPort2.Write(buffer, offset, count);
                }
                OnWritingData(null);
            }
            catch (System.Exception ex)
            {
                string logInfo = ex.Message;
                StackTrace st = new StackTrace(true);
                if (st.FrameCount > 1)
                {
                    StackFrame sf = st.GetFrame(1);
                    logInfo += sf.GetFileName();
                    logInfo += sf.GetMethod().Name;
                    logInfo += sf.GetFileLineNumber();
                }
                //日志记录
                //Dispatcher.gs_blackBoxFile.WriteFile(100, "4:" + logInfo);
                throw new Exception("写串口:" + logInfo);
            }
        }
        public void Write(char[] buffer, int offset, int count)
        {
            try
            {
                if (serialPort1.IsOpen)
                {
                    serialPort1.Write(buffer, offset, count);
                }
                OnWritingData(null);
            }
            catch (System.Exception ex)
            {
                string logInfo = ex.Message;
                StackTrace st = new StackTrace(true);
                if (st.FrameCount > 1)
                {
                    StackFrame sf = st.GetFrame(1);
                    logInfo += sf.GetFileName();
                    logInfo += sf.GetMethod().Name;
                    logInfo += sf.GetFileLineNumber();
                }
                //日志记录
                //Dispatcher.gs_blackBoxFile.WriteFile(100, "3:" + logInfo);
                throw new Exception("写串口:" + logInfo);
            }
        }
        public void Open()
        {
            try
            {
                if (!serialPort1.IsOpen)
                {
                    serialPort1.Open();

                    if (thread != null)
                    {
                        if (thread.ThreadState == System.Threading.ThreadState.Suspended)
                            thread.Resume();
                        else if (thread.ThreadState == System.Threading.ThreadState.Stopped || thread.ThreadState == System.Threading.ThreadState.Unstarted)
                        {
                            thread.Start();
                        }
                    }
                    else
                    {
                        thread = new Thread(ThreadHandler);
                        thread.Start();
                    }
                }
            }
            catch (System.Exception ex)
            {
                string logInfo = ex.Message;
                StackTrace st = new StackTrace(true);
                if (st.FrameCount > 1)
                {
                    StackFrame sf = st.GetFrame(1);
                    logInfo += sf.GetFileName();
                    logInfo += sf.GetMethod().Name;
                    logInfo += sf.GetFileLineNumber();
                }
                //日志记录
                //Dispatcher.gs_blackBoxFile.WriteFile(100, "2:" + logInfo);
                throw new Exception("打开串口:" + logInfo);
            }
        }
        public void Open1()
        {
            try
            {
                if (!serialPort2.IsOpen)
                {
                    serialPort2.Open();

                    if (thread != null)
                    {
                        if (thread.ThreadState == System.Threading.ThreadState.Suspended)
                            thread.Resume();
                        else if (thread.ThreadState == System.Threading.ThreadState.Stopped || thread.ThreadState == System.Threading.ThreadState.Unstarted)
                        {
                            thread.Start();
                        }
                    }
                    else
                    {
                        thread = new Thread(ThreadHandler);
                        thread.Start();
                    }
                }
            }
            catch (System.Exception ex)
            {
                string logInfo = ex.Message;
                StackTrace st = new StackTrace(true);
                if (st.FrameCount > 1)
                {
                    StackFrame sf = st.GetFrame(1);
                    logInfo += sf.GetFileName();
                    logInfo += sf.GetMethod().Name;
                    logInfo += sf.GetFileLineNumber();
                }
                //日志记录
                //Dispatcher.gs_blackBoxFile.WriteFile(100, "2:" + logInfo);
                throw new Exception("打开串口:" + logInfo);
            }
        }
        public void Close()
        {
            try
            {
                serialPort2.Close();
                serialPort1.Close();
                if (thread != null && thread.ThreadState == System.Threading.ThreadState.Running)
                {
                    if (done.WaitOne(500))
                    {
                        thread.Suspend();
                    }
                    thread.Suspend();
                }

                receiveBuffer.Clear();
            }
            catch (System.Exception ex)
            {
                string logInfo = ex.Message;
                StackTrace st = new StackTrace(true);
                if (st.FrameCount > 1)
                {
                    StackFrame sf = st.GetFrame(1);
                    logInfo += sf.GetFileName();
                    logInfo += sf.GetMethod().Name;
                    logInfo += sf.GetFileLineNumber();
                }
                //日志记录
                //Dispatcher.gs_blackBoxFile.WriteFile(100, "1:" + logInfo);
                throw new Exception("关闭串口:" + logInfo);
            }
        }
        public SerialPort serialPort1;
        public SerialPort serialPort2;
        public List<byte> receiveBuffer = new List<byte>(2046);
        private static object portLock = new object();
        public delegate void MatchedParityHandler(object sender, DataEventArgs e);
        public event MatchedParityHandler MatchedParity;
        private Thread thread;
        private void OnMatchedParity(object sender, DataEventArgs e)
        {
            if (MatchedParity != null)
            {
                MatchedParity(sender, e);
            }
        }
        public class MyData
        {
            public object sender;
            public DataEventArgs e;
        }
        public void OnMatchParity(object o)
        {
            MyData data = o as MyData;
            Console.ForegroundColor = ConsoleColor.Yellow;
            //Console.WriteLine("所在线程ID:" + Thread.CurrentThread.ManagedThreadId.ToString() + "类型:" + data.e.type + "  " + Byte2HexString(data.e.data));
            Console.ResetColor();

            OnMatchedParity(data.sender, data.e);
            Thread.Sleep(500);//加延时是为了避免解析后有过长的操作占用系统资源
        }
        public static string Byte2HexString(byte[] buffer)
        {
            StringBuilder builder = new StringBuilder(buffer.Length * 3);
            foreach (byte data in buffer)
            {
                builder.Append(Convert.ToString(data, 16).PadLeft(2, '0').PadRight(3, ' '));
            }
            return builder.ToString().ToUpper();
        }
        /// <summary>
        /// 波特率为57600
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void serialPort2_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            byte[] buffer = new byte[serialPort2.BytesToRead];
            serialPort2.Read(buffer, 0, buffer.Length);
            OnReadingData(null);
            Console.ForegroundColor = ConsoleColor.Green;
            //Console.WriteLine("接收数据:" + Byte2HexString(buffer));
            Console.ResetColor();
            lock (portLock)
            {
                receiveBuffer.AddRange(buffer);
            }
            //Console.WriteLine("缓冲区数据:" + Byte2HexString(receiveBuffer.ToArray()));
        }
        /// <summary>
        /// 波特率为9600
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            byte[] buffer = new byte[serialPort1.BytesToRead];
            serialPort1.Read(buffer, 0, buffer.Length);
            OnReadingData(null);
            Console.ForegroundColor = ConsoleColor.Green;
            //Console.WriteLine("接收数据:" + Byte2HexString(buffer));
            Console.ResetColor();
            lock (portLock)
            {
                receiveBuffer.AddRange(buffer);
            }
            //Console.WriteLine("缓冲区数据:" + Byte2HexString(receiveBuffer.ToArray()));
        }
        public static byte XOR_Check(byte[] buffer, int count, int index = 0)
        {
            byte b = 0;
            for (int i = 0; i < count; i++)
            {
                b ^= buffer[index + i];
            }
            return b;
        }
        /// <summary>
        /// 整理数据缓存表
        /// </summary>
        /// <param name="bufferList">数据缓存表</param>
        /// <param name="targetByte">目标字节</param>
        private static void ArrangeBuffer(List<byte> bufferList, byte targetByte)
        {
            int delBytes = 1;

            for (int i = 1; i < bufferList.Count; i++)
            {
                if (bufferList[i] == targetByte)
                {
                    break;
                }
                else
                {
                    delBytes++;
                }
            }

            try
            {
                lock (portLock)
                {
                    bufferList.RemoveRange(0, delBytes);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private byte[] DataHander(out int type)
        {
            type = -1;
            int framelength = 10;           // 帧长度
            byte frameHead = 0x10;          // 帧头
            byte frameTaig = 0x03;
            List<byte> frameFuncList = new List<byte> { 0x41, 0x42, 0x46, 0x32, 0x33, 0x35, 0x38 };

            if (receiveBuffer.Count >= framelength)
            {
                #region MyRegion
                if ((receiveBuffer[0] == frameHead) && (receiveBuffer.Count >= 14) && frameFuncList.Contains(receiveBuffer[1]))
                {
                    // 二代协议
                    byte checkSum = 0x00;
                    for (int i = 0; i < 12; i++)
                    {
                        checkSum ^= receiveBuffer[i];
                    }
                    if (checkSum == receiveBuffer[12] && frameTaig == receiveBuffer[13])
                    {
                        byte[] data = new byte[14];
                        for (int i = 0; i < 14; i++)
                        {
                            data[i] = receiveBuffer[i];
                        }
                        lock (portLock)
                        {
                            receiveBuffer.RemoveRange(0, 14);
                        }
                        type = 1;
                        return data;
                    }
                    else
                    {
                        ArrangeBuffer(receiveBuffer, frameHead);
                    }
                }
                else if ((receiveBuffer[0] == frameHead) && (receiveBuffer.Count >= 10) && receiveBuffer[1] == 0x62)
                {
                    byte checkSum = 0x00;
                    for (int i = 0; i < 8; i++)
                    {
                        checkSum ^= receiveBuffer[i];
                    }
                    if (checkSum == receiveBuffer[8] && frameTaig == receiveBuffer[9])
                    {
                        byte[] data = new byte[10];
                        for (int i = 0; i < 10; i++)
                        {
                            data[i] = receiveBuffer[i];
                        }
                        lock (portLock)
                        {
                            receiveBuffer.RemoveRange(0, 10);
                        }
                        type = 1;
                        return data;
                    }
                    else
                    {
                        ArrangeBuffer(receiveBuffer, frameHead);
                    }
                }
                else
                {
                    ArrangeBuffer(receiveBuffer, frameHead);
                }
                #endregion
            }
            return new byte[1] { 0 };
        }
        AutoResetEvent done = new AutoResetEvent(true);//在处理数据的时候不允许被打断（只有在线程正常挂起时有效）
        public void ThreadHandler()
        {
            while (true)
            {
                if (serialPort1.IsOpen || serialPort2.IsOpen)//
                {
                    if (receiveBuffer.Count > 0)
                    {
                        done.Reset();
                        byte[] buffer;
                        int type;
                        buffer = DataHander(out type);
                        DataEventArgs e = new DataEventArgs();
                        e.type = type;
                        e.data = buffer;
                        if (type != -1)
                        {
                            MyData data = new MyData();
                            data.sender = this;
                            data.e = e;
                            Thread t = new Thread(OnMatchParity);
                            t.Start(data);
                        }
                    }
                }
                Thread.Sleep(20);
                done.Set();
            }
        }
    }
}
