﻿using CasunCommon.SystemLog;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Xml;

namespace Pms.UserEvent.Model
{
    public class UDPCommunication : CommunicationBase
    {
        private string localIP;
        public List<byte> receiveBuffer = new List<byte>(2046);
        private static object portLock = new object();
        public delegate void MatchedParityHandler(object sender, DataEventArgs e);
        public event MatchedParityHandler MatchedParity;
        List<IPAddress> recieveIPs = new List<IPAddress>();
        // ConcurrentDictionary<IPAddress, string> recieveIpPort = new ConcurrentDictionary<IPAddress, string>();
        public string LocalIP
        {
            get
            {
                if (string.IsNullOrEmpty(localIP))
                {
                    return Dns.GetHostAddresses(Dns.GetHostName())[0].ToString();
                }
                return localIP;
            }
            set
            {
                string str = @"^\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}$";
                if (Regex.IsMatch(value, str))
                {
                    localIP = value;
                }
                else
                {
                    throw new Exception("IP地址格式错误");
                }
            }
        }
        private int port;

        public int Port
        {
            get
            {
                if (port <= 0)
                    return 10000;
                return port;
            }
            set { port = value; }
        }
        private int scanTime;

        public int ScanTime
        {
            get
            {
                if (scanTime <= 0)
                    return 10;
                return scanTime;
            }
            set { scanTime = value; }
        }
        public override object GetTag()
        {
            return this;
        }
        private UdpClient udpServer;
        private Thread thread;
        private Thread thread1;

        public UDPCommunication(string ip, int port, int scanTime = 0)
        {
            LocalIP = ip;
            Port = port;
            ScanTime = scanTime;
            IsOpen = false;
        }
        AutoResetEvent done = new AutoResetEvent(true);
        public static string ByteToHexString(byte[] buffer)
        {
            StringBuilder builder = new StringBuilder(buffer.Length * 3);
            foreach (byte data in buffer)
            {
                builder.Append(Convert.ToString(data, 16).PadLeft(2, '0').PadRight(3, ' '));
            }
            return builder.ToString().ToUpper();
        }
        bool isChange = false;

        private void ReceiveThread()
        {
            IPEndPoint remotePoint = new IPEndPoint(IPAddress.Any, 0);
            while (true)
            {
                if (IsOpen && udpServer != null)
                {
                    ReceivedEventArgs e1 = new ReceivedEventArgs();
                    try
                    {
                        e1.buffer = udpServer.Receive(ref remotePoint);

                        if (remotePoint.Address.ToString() != LocalIP)
                        {
                            if (!recieveIPs.Contains(remotePoint.Address))
                            {
                                //recieveIpPort.TryAdd(remotePoint.Address, remotePoint.Port.ToString());
                                recieveIPs.Add(remotePoint.Address);
                            }
                            string runMsg = ByteToHexString(e1.buffer);
                            receiveBuffer.AddRange(e1.buffer);
                            if (e1.buffer[5] == 0x00 && e1.buffer[6] == 0x00 && e1.buffer[7] == 0x00 && e1.buffer[8] == 0x00)
                            {
                                string AlarmStatus = "[open]";
                                if (e1.buffer[4] == 0x00)
                                {
                                    AlarmStatus = "[close]";
                                }
                                CLog.Instance.WriteLog("报警灯回复信息", $"回复信息:【{runMsg}】[{AlarmStatus}]");
                            }
                            //日志记录
                            //gs_blackBoxFile.WriteFile(10000, remotePoint.Address + ";" + ByteToHexString(e1.buffer));
                        }
                    }
                    catch (System.Exception ex)
                    {
                        Console.WriteLine(ex.Message);

                    }
                    Thread.Sleep(ScanTime);
                    done.Set();
                }
            }
        }
        private void ReceiveThread2()
        {
            IPEndPoint remotePoint = new IPEndPoint(IPAddress.Any, 0);
            while (true)
            {
                if (receiveBuffer.Count > 0)
                {
                    done.Reset();
                    byte[] buffer;
                    int type;
                    buffer = DataHander(out type);
                    DataEventArgs e = new DataEventArgs();
                    e.type = type;
                    e.data = buffer;
                    if (type != -1)
                    {
                        MyData data = new MyData();
                        data.sender = this;
                        data.e = e;
                        Thread t = new Thread(OnMatchParity);
                        t.Start(data);
                    }
                }
                Thread.Sleep(20);
            }
        }
        public event EventHandler ReadingData;
        public event EventHandler WritingData;
        void OnReadingData(EventArgs e)
        {
            if (ReadingData != null)
            {
                ReadingData(this, e);
            }
        }
        void OnWritingData(EventArgs e)
        {
            if (WritingData != null)
            {
                WritingData(this, e);
            }
        }
        public void OnMatchedParity(object sender, DataEventArgs e)
        {
            if (MatchedParity != null)
            {
                MatchedParity(sender, e);
            }
        }
        public void OnMatchParity(object o)
        {
            MyData data = o as MyData;
            Console.ForegroundColor = ConsoleColor.Yellow;
            //Console.WriteLine("所在线程ID:" + Thread.CurrentThread.ManagedThreadId.ToString() + "类型:" + data.e.type + "  " + Byte2HexString(data.e.data));
            Console.ResetColor();

            OnMatchedParity(data.sender, data.e);
            Thread.Sleep(500);//加延时是为了避免解析后有过长的操作占用系统资源
        }
        private class MyData
        {
            public object sender;
            public DataEventArgs e;
        }
        public static byte XOR_Check(byte[] buffer, int count, int index = 0)
        {
            byte b = 0;
            for (int i = 0; i < count; i++)
            {
                b ^= buffer[index + i];
            }
            return b;
        }
        /// <summary>
        /// 整理数据缓存表
        /// </summary>
        /// <param name="bufferList">数据缓存表</param>
        /// <param name="targetByte">目标字节</param>
        private static void ArrangeBuffer(List<byte> bufferList, byte targetByte)
        {
            int delBytes = 1;

            for (int i = 1; i < bufferList.Count; i++)
            {
                if (bufferList[i] == targetByte)
                {
                    break;
                }
                else
                {
                    delBytes++;
                }
            }
            try
            {
                lock (portLock)
                {
                    bufferList.RemoveRange(0, delBytes);
                }
            }
            catch (Exception ex)
            {
                //MessageBox.Show(ex.Message);
            }
        }
        private byte[] DataHander(out int type)
        {
            type = -1;
            int framelength = 10;           // 帧长度
            byte frameHead = 0x10;          // 帧头
            byte frameTaig = 0x03;
            List<byte> frameFuncList = new List<byte> { 0x41, 0x42, 0x46, 0x32, 0x33, 0x35, 0x38, 0x37 };

            if (receiveBuffer.Count >= framelength)
            {
                #region MyRegion
                if ((receiveBuffer[0] == frameHead) && (receiveBuffer.Count >= 14) && frameFuncList.Contains(receiveBuffer[1]))
                {
                    // 二代协议
                    byte checkSum = 0x00;
                    for (int i = 0; i < 12; i++)
                    {
                        checkSum ^= receiveBuffer[i];
                    }
                    if (checkSum == receiveBuffer[12] && frameTaig == receiveBuffer[13])
                    {
                        byte[] data = new byte[14];
                        for (int i = 0; i < 14; i++)
                        {
                            data[i] = receiveBuffer[i];
                        }
                        lock (portLock)
                        {
                            receiveBuffer.RemoveRange(0, 14);
                        }
                        type = 3;
                        return data;
                    }
                    else
                    {
                        ArrangeBuffer(receiveBuffer, frameHead);
                    }
                }
                else if ((receiveBuffer[0] == frameHead) && (receiveBuffer.Count >= 10) && receiveBuffer[1] == 0x62)
                {
                    byte checkSum = 0x00;
                    for (int i = 0; i < 8; i++)
                    {
                        checkSum ^= receiveBuffer[i];
                    }
                    if (checkSum == receiveBuffer[8] && frameTaig == receiveBuffer[9])
                    {
                        byte[] data = new byte[10];
                        for (int i = 0; i < 10; i++)
                        {
                            data[i] = receiveBuffer[i];
                        }
                        lock (portLock)
                        {
                            receiveBuffer.RemoveRange(0, 10);
                        }
                        type = 3;
                        return data;
                    }
                    else
                    {
                        ArrangeBuffer(receiveBuffer, frameHead);
                    }
                }
                else
                {
                    ArrangeBuffer(receiveBuffer, frameHead);
                }
                #endregion
            }
            return new byte[1] { 0 };
        }
        public override void SendData(byte[] buffer, object o)
        {
            IPEndPoint ip = o as IPEndPoint;
            if (ip == null)
            {
                string ipStr = LocalIP.Substring(0, LocalIP.LastIndexOf('.') + 1);
                try
                {
                    if (IsOpen)
                    {
                        if (buffer[5] == 0x99 && buffer[6] == 0x99 && buffer[7] == 0x99 && buffer[8] == 0x99)
                        {
                            StringBuilder sb = new StringBuilder();
                            sb.Append($"select * from T_AlarmTable");
                            DataTable dt1 = ConDataBase_PMS.reDt(sb.ToString());
                            if (dt1 != null && dt1.Rows != null && dt1.Rows.Count > 0)
                            {
                                IPAddress tmpIp;
                                if (IPAddress.TryParse(dt1.Rows[0]["_IP"].ToString(), out tmpIp))
                                {
                                    string fs = IPAddress.Parse(dt1.Rows[0]["_IP"].ToString()).ToString();
                                }
                                string AlarmStatus = "[open]";
                                if (buffer[4] == 0x00)
                                {
                                    AlarmStatus = "[close]";
                                }
                                int _port = int.Parse(dt1.Rows[0]["_Port"].ToString());
                                udpServer.Send(buffer, buffer.Length, new IPEndPoint(tmpIp, _port));
                                CLog.Instance.WriteLog("报警灯数据发送记录", $"报警灯IP:【{tmpIp}】报警灯Port:【{_port}】【{DateTime.Now.ToString("HH:mm:ss")}】发送:【{ByteToHexString(buffer)}】{AlarmStatus}");
                                return;
                            }
                        }
                        string runMsg = ByteToHexString(buffer);
                        //Console.WriteLine(string.Format("[{0,-15}:{1} {2}]发送:{3}", LocalIP, Port, DateTime.Now.ToString("HH:mm:ss"), ByteToHexString(buffer)));
                        CLog.Instance.WriteLog("叫料盒数据发送记录", $"叫料盒IP:【{LocalIP}】叫料盒Port:【{Port}】【{DateTime.Now.ToString("HH:mm:ss")}】发送:【{ByteToHexString(buffer)}】");
                        if (recieveIPs.Count < 0 && !(buffer[5] == 0x99 && buffer[6] == 0x99 && buffer[7] == 0x99 && buffer[8] == 0x99))
                        {
                            udpServer.Send(buffer, buffer.Length, new IPEndPoint(IPAddress.Parse("255.255.255.255"), Port));
                        }
                        else
                        {
                            if (!(buffer[5] == 0x99 && buffer[6] == 0x99 && buffer[7] == 0x99 && buffer[8] == 0x99))
                            {
                                foreach (IPAddress ipa in recieveIPs)
                                {
                                    udpServer.Send(buffer, buffer.Length, new IPEndPoint(ipa, Port));
                                }
                            }

                            //for (int i = 0; i < recieveIPs.Count; i++)
                            //{
                            //    for (int j = 0; j < 3; j++)
                            //    {
                            //        udpServer.Send(buffer, buffer.Length, new IPEndPoint(recieveIPs[i], Port));
                            //    }
                            //    string ClientPort = string.Empty;
                            //    recieveIpPort.TryGetValue(recieveIPs[i], out ClientPort);
                            //    CLog.Instance.WriteLog("叫料盒数据发送记录", $"叫料盒IP:【{recieveIPs[i]}】叫料盒Port:【{ClientPort}】发送数据:【{runMsg}】");
                            //    recieveIPs.RemoveAt(i);
                            //}
                        }
                    }
                }
                catch (System.Exception ex)
                {
                    throw new Exception(ex.Message);
                }
            }
            else
            {
                try
                {
                    if (IsOpen)
                    {
                        Console.WriteLine(string.Format("[{0,-15}:{1} {2}]发送:{3}", LocalIP, Port, DateTime.Now.ToString("HH:mm:ss"), ByteToHexString(buffer)));
                        udpServer.Send(buffer, buffer.Length, new IPEndPoint(ip.Address, Port));
                    }

                }
                catch (System.Exception ex)
                {
                    throw new Exception(ex.Message);
                }
            }
        }
        public override void Close()
        {
            if (udpServer != null)
            {
                udpServer.Close();
                udpServer = null;
            }
            if (thread != null && thread.ThreadState != ThreadState.Stopped)
            {
                if (done.WaitOne(500))
                {
                    thread.Abort();
                }
                thread = null;
            }

            IsOpen = false;
        }
        public override void Dispose()
        {
            if (udpServer != null)
                udpServer.Close();
            if (thread != null && thread.ThreadState != ThreadState.Stopped)
            {
                if (done.WaitOne(500))
                {
                    thread.Abort();
                }
                thread.Abort();
                thread = null;
            }
            base.Dispose();

        }

        public override void Open()
        {
            if (thread != null && udpServer != null)
            {
                if (thread.ThreadState == ThreadState.Suspended)
                {
                    thread.Resume();
                }
                else if (thread.ThreadState == ThreadState.Unstarted)
                {
                    thread.Start();
                }
            }
            else
            {
                IPAddress[] address = Dns.GetHostAddresses(Dns.GetHostName());
                for (int i = 0; i < address.Count(); i++)
                {

                    if (address[i].AddressFamily.ToString().ToLower() == "internetwork" && address[i].ToString() == LocalIP)
                    {
                        udpServer = new UdpClient(new IPEndPoint(IPAddress.Any, Port));
                        thread = new Thread(ReceiveThread);
                        thread.Start();
                        thread1 = new Thread(ReceiveThread2);
                        thread1.Start();
                        break;
                    }
                    else if (i == address.Count() - 1)
                    {
                        Console.WriteLine("UDP中IP地址不存在");
                        return;
                    }
                }

            }
            IsOpen = true;
        }

        private void InitIPConfig()
        {
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load("PMS\\Config\\IPAddress.xml");
            XmlNode xn = xmlDoc.SelectSingleNode("AGVS");//根节点
            XmlNodeList nodeList = xn.ChildNodes;//根节点下面的子节点
            foreach (XmlNode xnf in nodeList)//遍历所有子节点
            {
                XmlElement xe = (XmlElement)xnf;
                string AgvId = xe.ChildNodes[0].InnerText;//agv编号 
                XmlNodeList ipList = xe.ChildNodes[1].ChildNodes;
                foreach (XmlNode ip in ipList)
                {
                    XmlElement ipxe = (XmlElement)ip;
                    string ipStr = ip.InnerText;//IP地址 

                    if (IPCheck(ipStr))
                    {
                        if (!IpBuffer.Contains(ipStr))
                        {
                            IpBuffer.Add(ipStr);
                        }
                    }
                }
            }
        }
        private List<string> IpBuffer = new List<string>();
        public bool IPCheck(string ipAddress)
        {
            if (string.IsNullOrEmpty(ipAddress))
                return false;

            Regex validipregex = new Regex(@"^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$");
            return (ipAddress != "" && validipregex.IsMatch(ipAddress.Trim()));
        }
    }
}
