﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Pms.EventLibrary.TaskEvent;
using Pms.EventLibrary.TaskEventArgs;
using System.Windows.Forms;
using Pms.InternalInterface.Models;
using Pms.InternalInterface.Enums;
using Pms.UserEvent.SystemEvent;
using System.Data;
using Pms.UserEvent.Model;
using CasunCommon.SystemLog;

namespace Pms.UserEvent.OrderEvent
{
    public partial class COrderEvent : BaseOrderEvent
    {
        public COrderEvent() : base()
        { }
        /// <summary>
        /// 任务创建前触发此事件
        /// 设置 e.IsSuccessed = false 则任务不再执行 即任务异常
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderBeginCreate(object sender, P_OrderEventArgs e)
        {
        }

        /// <summary>
        /// 任务下发AGVS后触发
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderStarted(object sender, P_OrderEventArgs e)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append($" update  ");
            sb.Append($" MesOrderTask ");
            sb.Append($" set ");
            sb.Append($" actaionStatus=1 ");
            sb.Append($" where ");
            sb.Append($" taskChainCode='{e.OrderData.OrderID}' ");
            int flag = ConDataBase_PMS.sqlNonQuery(sb.ToString());
            if (flag > 0)
            {
                CLog.Instance.WriteLog("任务更新记录", $"OrderID:【{e.OrderData.OrderID}】actaionStatus:【1】");
            }
            else
            {
                CLog.Instance.WriteLog("任务更新记录", $"OrderID:【{e.OrderData.OrderID}】失败");
            }
            //int id = 0;
            //if (int.TryParse(e.OrderData.OrderID, out id))
            //{
            //    StringBuilder sb = new StringBuilder();
            //    sb.Append($" SELECT * FROM CallTask  ");
            //    sb.Append($" where ");
            //    sb.Append($" Id={id} ");
            //    DataTable dt = ConDataBase_WMS.reDt(sb.ToString());
            //    if (dt != null && dt.Rows != null && dt.Rows.Count > 0)
            //    {
            //        sb.Clear();
            //        sb.Append($" update  ");
            //        sb.Append($" CallTask  ");
            //        sb.Append($" set  ");
            //        sb.Append($" TaskStatus='2'  ");
            //        sb.Append($" where Id={id} ");
            //        int flag = ConDataBase_WMS.sqlNonQuery(sb.ToString());
            //        if (flag > 0)
            //            CLog.Instance.WriteLog("Pad任务更新记录", $"OrderID:【{e.OrderData.OrderID}】TaskStatus:【2】");
            //    }
            //}
        }
        /// <summary>
        /// AGVS回调 小车给AGV后触发
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderAssignedCarrier(object sender, P_OrderEventArgs e)
        {

        }
        // 当前版本不会触发
        protected override void OrderExecuting(object sender, P_OrderEventArgs e)
        {
        }

        /// <summary>
        /// 任务异常不能执行时，会触发此事件
        /// 此处添加可以 自动处理的异常 的处理代码
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderException(object sender, P_OrderEventArgs e)
        {
            if (e.OrderData.OrderID.StartsWith("T"))
            {
                try
                {
                    bool succeed = false;
                    ReportMaterialLocationResponse reportMaterialLocationResponse = SCL_DMS_WebServiceHelper.ReportTaskStatus(new ReportTaskStatusRequest() { TaskId = e.OrderData.OrderID, Status = "取消", MaterlalId = e.OrderData.Remark, AGVId = e.OrderData.Carrier.ToString() });
                    if (reportMaterialLocationResponse.resCode == "0")
                    {
                        succeed = true;
                    }
                    CLog.Instance.WriteLog("AGV上报任务取消", $"返回resCode:【{reportMaterialLocationResponse.resCode}】，resMsg:【{reportMaterialLocationResponse.resMsg}】");
                }
                catch(Exception ex)
                {
                    CLog.Instance.WriteLog("AGV上报任务取消", $"异常:【{ex.ToString()}】");
                }
            }
        }


        /// <summary>
        /// 调用取消接口后触发此事件
        /// 任务取消成功与否 都会触发此事件
        /// e.IsCancelSuccess = True ： 取消成功
        /// e.IsCancelSuccess = false： 取消失败
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderCanceled(object sender, P_OrderEventArgs e)
        {
            if (e.OrderData.OrderID.StartsWith("T"))
            {
                bool succeed = false;
                ReportMaterialLocationResponse reportMaterialLocationResponse = SCL_DMS_WebServiceHelper.ReportTaskStatus(new ReportTaskStatusRequest() { TaskId = e.OrderData.OrderID, Status = "取消", MaterlalId = e.OrderData.Remark, AGVId = e.OrderData.Carrier.ToString() });
                if (reportMaterialLocationResponse.resCode == "0")
                {
                    succeed = true;
                }
                CLog.Instance.WriteLog("AGV上报任务取消", $"返回resCode:【{reportMaterialLocationResponse.resCode}】，resMsg:【{reportMaterialLocationResponse.resMsg}】");
            }
        }

        // 当前版本不会触发
        protected override void OrderChanged(object sender, P_OrderEventArgs e)
        {
        }

        /// <summary>
        /// 任务执行完成会触发此事件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderFinished(object sender, P_OrderEventArgs e)
        {
            //int id = 0;
            //if (int.TryParse(e.OrderData.OrderID, out id))
            //{
            StringBuilder sb = new StringBuilder();
            sb.Append($" update  ");
            sb.Append($" MesOrderTask ");
            sb.Append($" set ");
            sb.Append($" actaionStatus=2 ");
            sb.Append($" where ");
            sb.Append($" taskChainCode='{e.OrderData.OrderID}' ");
            int flag = ConDataBase_PMS.sqlNonQuery(sb.ToString());
            if (flag > 0)
            {
                CLog.Instance.WriteLog("任务更新记录", $"OrderID:【{e.OrderData.OrderID}】actaionStatus:【2】");
            }
            else
            {
                CLog.Instance.WriteLog("任务更新记录", $"OrderID:【{e.OrderData.OrderID}】失败");
            }
            //    sb.Append($" SELECT * FROM CallTask  ");
            //    sb.Append($" where ");
            //    sb.Append($" Id={id} ");
            //    DataTable dt = ConDataBase_WMS.reDt(sb.ToString());
            //    if (dt != null && dt.Rows != null && dt.Rows.Count > 0)
            //    {
            //        sb.Clear();
            //        sb.Append($" SELECT * FROM   ");
            //        sb.Append($" t_wh_locationProperty ");
            //        sb.Append($" where ");
            //        sb.Append($" Attribute1='{e.OrderData.PosArr[0]}' ");
            //        DataTable dt_Area = ConDataBase_WMS.reDt(sb.ToString());
            //        string Area = dt_Area.Rows[0]["Area"].ToString();
            //        sb.Clear();
            //        sb.Append($" update  ");
            //        sb.Append($" CallTask  ");
            //        sb.Append($" set  ");
            //        sb.Append($" TaskStatus='3'  ");
            //        sb.Append($" where Id={id} ");

            //        sb.Append($" update  ");
            //        sb.Append($" t_wh_locationProperty ");
            //        sb.Append($" set ");
            //        sb.Append($" Status=1 ");
            //        sb.Append($" where ");
            //        sb.Append($" Area='{Area}' ");
            //        int flag = ConDataBase_WMS.sqlNonQuery(sb.ToString());
            //        if (flag > 0)
            //        {
            //            CLog.Instance.WriteLog("Pad任务更新记录", $"OrderID:【{id}】TaskStatus:【4】 更新库位状态为【1】");
            //        }
            //        else
            //        {
            //            CLog.Instance.WriteLog("Pad任务更新记录", $"OrderID:【{id}】TaskStatus:【4】 更新库位状态为【1】任务结束更新失败{sb.ToString()}");
            //        }
            //    }
            //}
        }

        /// <summary>
        /// 如果配置了步骤行为60035 则需要处理此事件
        /// 用来决定任务AGV小车是否可以 释放事件  
        /// e.IsSuccessed = true; 则 系统认为可以释放小车
        /// e.IsSuccessed = false; 则 系统认为不可以 释放小车
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderCarrierNeedRelease(object sender, P_OrderEventArgs e)
        {



        }
        /// <summary>
        /// 任务模板 选择时触发此事件
        /// e.OrderData.BookTemplateID_RW = 值
        /// 设置 e.IsSuccessed = false 则任务不再执行 即任务异常
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OrderTemplateSelecting(object sender, P_OrderEventArgs e)
        {

        }

        /// <summary>
        /// 系统重启时处理历史任务时触发
        /// 设置 e.ExecuteCode 值选择不同的处理方式
        /// e.ExecuteCode 值
        /// 0 ：默认处理 方法 已开始的任务置为异常；未开始的任务 ，如果隔天直接删除 ，未隔天 则重新开始
        /// 1 ：置为异常
        /// 2 ：重新开始 
        /// 3 ：直接删除
        /// 其余值 默认处理方法
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void RestartHisOrderExecute(object sender, P_RestartHisOrderExecuteEventArgs e)
        {
            e.ExecuteCode = 3;
        }
    }
}

