﻿using Newtonsoft.Json;
using PmsWebApi;
using System;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;

namespace WabApiTest
{
    public class CPmsApi
    {
        public string m_URL;

        public enum RequestMethod
        {
            Get,
            Post,
            Send,
            Put
        }

        private enum EbizCode
        {
            Default,
            OrderAdd,
            MidOrderAdd,
            OrderCancel,
            GetOrder,
            GetCurrentOrder,
            GetCarrier,
            GetAllCarrier,
        }

        private enum EWebLogLevel
        {
            Error,
            Msg,
        }

        public CPmsApi()
        {
        }

        public bool MidOrderAddSync(RequestMidOrder sRequestMidOrder,out string requestJson ,out string res,out string strMsg)
        {
            EbizCode bizCode = EbizCode.MidOrderAdd;
            string strUrl = m_URL + bizCode.ToString();
            requestJson = JsonConvert.SerializeObject(sRequestMidOrder);

            return Request(strUrl, bizCode, requestJson, out res, out strMsg);
        }

        public bool OrderAddSync(RequestOrderInfo sRequestOrderInfo, out string requestJson, out string res, out string strMsg)
        {
            EbizCode bizCode = EbizCode.OrderAdd;
            string strUrl = m_URL + bizCode.ToString();
            requestJson = JsonConvert.SerializeObject(sRequestOrderInfo);

            return Request(strUrl, bizCode, requestJson, out res, out strMsg);
        }

        public bool OrderCancelSync(string OrderID, out string requestJson, out string res, out string strMsg)
        {
            EbizCode bizCode = EbizCode.OrderCancel;
            string strUrl = m_URL + bizCode.ToString();
            requestJson = JsonConvert.SerializeObject(OrderID);

            return Request(strUrl, bizCode, requestJson,out res,out strMsg);
        }

        public bool GetOrderSync(string OrderID, out string requestJson, out string res, out string strMsg)
        {
            EbizCode bizCode = EbizCode.GetOrder;
            string strUrl = m_URL + bizCode.ToString();
            requestJson = JsonConvert.SerializeObject(OrderID);

            return Request(strUrl, bizCode, requestJson, out res, out strMsg);
        }

        public bool GetCurrentOrderSync(out string requestJson, out string res, out string strMsg)
        {
            EbizCode bizCode = EbizCode.GetCurrentOrder;
            string strUrl = m_URL + bizCode.ToString();
            requestJson = "";

            return Request(strUrl, bizCode, requestJson, out res, out strMsg);
        }

        public bool GetCarrierSync(int CarrierID, out string requestJson, out string res, out string strMsg)
        {
            EbizCode bizCode = EbizCode.GetCarrier;
            string strUrl = m_URL + bizCode.ToString();
            requestJson = JsonConvert.SerializeObject(CarrierID.ToString());

            return Request(strUrl, bizCode, requestJson, out res, out strMsg);
        }

        public bool GetAllCarrierSync(out string requestJson, out string res, out string strMsg)
        {
            EbizCode bizCode = EbizCode.GetAllCarrier;
            string strUrl = m_URL + bizCode.ToString();
            requestJson = "";

            return Request(strUrl, bizCode, requestJson, out res, out strMsg);
        }


        private bool Request(string strUrl, EbizCode bizCode, string requestJson, out string res, out string strMsg)
        {
            res = "";
            try
            {
                res = Request(strUrl, requestJson, RequestMethod.Post);
            }
            catch (Exception ex)
            {
                strMsg = string.Format("Error_{0}：Info = 【{1}】,Url = 【{2}】\r\n ErrorMsg = 【{3}】"
                    , bizCode.ToString()
                    , requestJson
                    , strUrl
                    , ex.Message
                    );
                return false;
            }

            if (res == "")
            {
                strMsg = string.Format("Fail_{0}：Info = 【{1}】,Url = 【{2}】,FailMsg = 【Network anomaly】"
                    , bizCode.ToString()
                    , requestJson
                    , strUrl
                    );
                return false;
            }

            Result cres = JsonConvert.DeserializeObject<Result>(res);

            if (cres.resCode != 0) // 0：成功
            {
                strMsg = string.Format("Fail_{0}：Info = 【{1}】,Url = 【{2}】,resCode = 【{3}】, resMsg = 【{4}】"
                     , bizCode.ToString()
                     , requestJson
                     , strUrl
                     , cres.resCode
                     , cres.resMsg
                     );
                return false;
            }
            else
            {
                strMsg = "Success";
                return true;
            }
        }

        private string Request(string url, string data = "", RequestMethod requestMethod = RequestMethod.Get)
        {
            try
            {
                HttpClient http = new HttpClient(new HttpClientHandler()
                {
                    //启用压缩,节省流量
                    AutomaticDecompression = DecompressionMethods.GZip
                });

                HttpResponseMessage response = null;
                switch (requestMethod)
                {
                    case RequestMethod.Send:
                        HttpRequestMessage hrm = new HttpRequestMessage(HttpMethod.Get, url);
                        HttpContent hc = new StringContent(data);
                        hc.Headers.ContentType = new MediaTypeHeaderValue("application/x-www-form-urlencoded");
                        hrm.Content = hc;
                        response = http.SendAsync(hrm).Result;
                        break;
                    case RequestMethod.Put:
                        HttpContent putContent = new StringContent(data);
                        putContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                        response = http.PutAsync(url, putContent).Result;
                        break;
                    case RequestMethod.Get:
                        http.DefaultRequestHeaders.Add("ContentType", "application/x-www-form-urlencoded");
                        response = http.GetAsync(url).Result;
                        break;
                    case RequestMethod.Post:
                        HttpContent httpContent = new StringContent(data);
                        httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                        response = http.PostAsync(url, httpContent).Result;
                        break;
                    default:
                        break;
                }
                var res = response?.Content.ReadAsStringAsync().Result;

                return res;
            }
            catch (Exception ex)
            {
                string strLog = "Request Exception:" + ex.Message;
                WriteWebLog(EbizCode.Default, EWebLogLevel.Error, strLog);
                return string.Empty;
            }
        }

        private void WriteWebLog(EbizCode bizCode, EWebLogLevel logLevel, string strLog)
        {
            string strLogName = "Web";

            switch (logLevel)
            {
                case EWebLogLevel.Error:
                    strLogName += "Error";
                    break;
            }
        }
    }
}

